diff -rbNu linux-2.4.31/Makefile linux-2.4.31-rtl3.1/Makefile
--- linux-2.4.31/Makefile	2005-06-01 02:56:56.000000000 +0200
+++ linux-2.4.31-rtl3.1/Makefile	2008-01-17 09:21:47.177022500 +0100
@@ -1,7 +1,7 @@
 VERSION = 2
 PATCHLEVEL = 4
 SUBLEVEL = 31
-EXTRAVERSION =
+EXTRAVERSION =-rtl-3.1
 
 KERNELRELEASE=$(VERSION).$(PATCHLEVEL).$(SUBLEVEL)$(EXTRAVERSION)
 
diff -rbNu linux-2.4.31/arch/i386/config.in linux-2.4.31-rtl3.1/arch/i386/config.in
--- linux-2.4.31/arch/i386/config.in	2004-11-17 12:54:21.000000000 +0100
+++ linux-2.4.31-rtl3.1/arch/i386/config.in	2008-01-17 09:18:44.883166500 +0100
@@ -2,11 +2,22 @@
 # For a description of the syntax of this configuration file,
 # see Documentation/kbuild/config-language.txt.
 #
-mainmenu_name "Linux Kernel Configuration"
+mainmenu_name "RTLinux/GPL Kernel Configuration"
 
 define_bool CONFIG_X86 y
 define_bool CONFIG_SBUS n
 
+  
+mainmenu_option next_comment
+comment 'Enable RTLinux HAL'
+define_bool CONFIG_RTLINUX  y
+if [ "$CONFIG_RTLINUX" = "y" ]; then
+  define_bool CONFIG_RTL y
+fi
+bool 'TLB flush points in do_IRQ' CONFIG_IRQ_TLBFLUSH
+bool 'TLB flush points in net code' CONFIG_NET_TLBFLUSH
+endmenu
+  
 define_bool CONFIG_UID16 y
 
 mainmenu_option next_comment
diff -rbNu linux-2.4.31/arch/i386/kernel/Makefile linux-2.4.31-rtl3.1/arch/i386/kernel/Makefile
--- linux-2.4.31/arch/i386/kernel/Makefile	2005-04-04 03:42:19.000000000 +0200
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/Makefile	2008-01-17 09:18:44.883166500 +0100
@@ -30,6 +30,7 @@
 endif
 endif
 
+obj-$(CONFIG_RTLINUX)           += rtlinux.o
 obj-$(CONFIG_MCA)		+= mca.o
 obj-$(CONFIG_MTRR)		+= mtrr.o
 obj-$(CONFIG_X86_MSR)		+= msr.o
diff -rbNu linux-2.4.31/arch/i386/kernel/apic.c linux-2.4.31-rtl3.1/arch/i386/kernel/apic.c
--- linux-2.4.31/arch/i386/kernel/apic.c	2004-11-17 12:54:21.000000000 +0100
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/apic.c	2008-01-17 09:18:44.898752500 +0100
@@ -785,9 +785,9 @@
 
 	spin_lock_irqsave(&i8253_lock, flags);
 
-	outb_p(0x00, 0x43);
-	count = inb_p(0x40);
-	count |= inb_p(0x40) << 8;
+	outb(0x00, 0x43);
+	count = inb(0x40);
+	count |= inb(0x40) << 8;
 
 	spin_unlock_irqrestore(&i8253_lock, flags);
 
diff -rbNu linux-2.4.31/arch/i386/kernel/entry.S linux-2.4.31-rtl3.1/arch/i386/kernel/entry.S
--- linux-2.4.31/arch/i386/kernel/entry.S	2003-06-13 16:51:29.000000000 +0200
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/entry.S	2008-01-17 09:18:44.914338500 +0100
@@ -184,6 +184,7 @@
 
 
 ENTRY(ret_from_fork)
+	sti
 	pushl %ebx
 	call SYMBOL_NAME(schedule_tail)
 	addl $4, %esp
@@ -197,11 +198,34 @@
  * but we want the default path for a system call return to
  * go as quickly as possible which is why some of this is
  * less clear than it otherwise should be.
+ *
+ * (c) Victor Yodaiken 1999
+ * RTLINUX_IRET emulates the turn on interrupts effect of
+ * iret since, under RTLinux we may have pended interrupts
+ * that will not be automatically enabled on an iret.
  */
+#ifdef CONFIG_RTLINUX
+#define RTLINUX_IRET 	\
+	movl  rtl_emulate_iret,%eax; \
+	testl %eax, %eax; \
+	je  1f;	\
+	call  *%eax;	\
+	1:
+#else
+#define RTLINUX_IRET
+#endif
 
 ENTRY(system_call)
 	pushl %eax			# save orig_eax
 	SAVE_ALL
+#ifdef CONFIG_RTLINUX
+	movl  rtl_syscall_intercept,%ebx
+	testl %ebx, %ebx
+	je  991f
+	call  *%ebx
+	movl ORIG_EAX(%esp), %eax
+991:
+#endif
 	GET_CURRENT(%ebx)
 	testb $0x02,tsk_ptrace(%ebx)	# PT_TRACESYS
 	jne tracesys
@@ -210,17 +234,21 @@
 	call *SYMBOL_NAME(sys_call_table)(,%eax,4)
 	movl %eax,EAX(%esp)		# save the return value
 ENTRY(ret_from_sys_call)
-	cli				# need_resched and signals atomic test
+	call *(SYMBOL_NAME(irq_control)+8) # cli from irq_control structure
 	cmpl $0,need_resched(%ebx)
 	jne reschedule
 	cmpl $0,sigpending(%ebx)
 	jne signal_return
+	call *(SYMBOL_NAME(irq_control)+12) # sti from irq_control structure
+	
 restore_all:
+	RTLINUX_IRET
 	RESTORE_ALL
 
 	ALIGN
 signal_return:
-	sti				# we can get here from an interrupt handler
+	call *(SYMBOL_NAME(irq_control)+12) # sti from irq_control structure
+	RTLINUX_IRET
 	testl $(VM_MASK),EFLAGS(%esp)
 	movl %esp,%eax
 	jne v86_signal_return
@@ -267,6 +295,12 @@
 	call SYMBOL_NAME(schedule)    # test
 	jmp ret_from_sys_call
 
+#define RTL_PUSH_VECTOR(vector,routine) \
+1:	cmpl $routine, %edi; \
+	jne 1f; \
+	pushl $vector; \
+	jmp 7f;
+
 ENTRY(divide_error)
 	pushl $0		# no error code
 	pushl $ SYMBOL_NAME(do_divide_error)
@@ -294,6 +328,39 @@
 	movl $(__KERNEL_DS),%edx
 	movl %edx,%ds
 	movl %edx,%es
+#ifdef CONFIG_RTLINUX
+	movl  rtl_exception_intercept,%eax
+	testl %eax, %eax
+	je  8f
+	RTL_PUSH_VECTOR(0,do_divide_error)
+	RTL_PUSH_VECTOR(1,do_debug)
+	RTL_PUSH_VECTOR(3,do_int3)
+	RTL_PUSH_VECTOR(4,do_overflow)
+	RTL_PUSH_VECTOR(5,do_bounds)
+	RTL_PUSH_VECTOR(6,do_invalid_op)
+	RTL_PUSH_VECTOR(8,do_double_fault)
+	RTL_PUSH_VECTOR(9,do_coprocessor_segment_overrun)
+	RTL_PUSH_VECTOR(10,do_invalid_TSS)
+	RTL_PUSH_VECTOR(11,do_segment_not_present)
+	RTL_PUSH_VECTOR(12,do_stack_segment)
+	RTL_PUSH_VECTOR(13,do_general_protection)
+	RTL_PUSH_VECTOR(14,do_page_fault)
+	RTL_PUSH_VECTOR(15,do_spurious_interrupt_bug)
+	RTL_PUSH_VECTOR(16,do_coprocessor_error)
+	RTL_PUSH_VECTOR(17,do_alignment_check)
+	RTL_PUSH_VECTOR(18,do_machine_check)
+	RTL_PUSH_VECTOR(19,do_simd_coprocessor_error)
+1:	push $-1
+7:	mov %ecx, %ebx			# ebx will not be corrupted by a C routine
+	call  *%eax
+	popl %ecx
+	mov %ebx, %ecx
+	testl %eax, %eax
+	je 8f
+	addl $8,%esp
+	RESTORE_ALL
+8:
+#endif
 	GET_CURRENT(%ebx)
 	call *%edi
 	addl $8,%esp
@@ -312,6 +379,21 @@
 ENTRY(device_not_available)
 	pushl $-1		# mark this as an int
 	SAVE_ALL
+#ifdef CONFIG_RTLINUX
+	movl  rtl_exception_intercept,%eax
+	testl %eax, %eax
+	je  8f
+	movl %esp,%edx
+	pushl $0		# no error code
+	pushl %edx
+	pushl $7		# vector
+	call  *%eax
+	addl $12,%esp
+	testl %eax, %eax
+	je 8f
+	RESTORE_ALL
+8:
+#endif
 	GET_CURRENT(%ebx)
 	movl %cr0,%eax
 	testl $0x4,%eax			# EM (math emulation bit)
diff -rbNu linux-2.4.31/arch/i386/kernel/i386_ksyms.c linux-2.4.31-rtl3.1/arch/i386/kernel/i386_ksyms.c
--- linux-2.4.31/arch/i386/kernel/i386_ksyms.c	2004-04-14 15:05:25.000000000 +0200
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/i386_ksyms.c	2008-01-17 09:18:44.929924500 +0100
@@ -182,6 +182,51 @@
 extern int is_sony_vaio_laptop;
 EXPORT_SYMBOL(is_sony_vaio_laptop);
 
+#ifdef CONFIG_RTLINUX
+#include <asm/mpspec.h>
+#ifdef CONFIG_X86_IO_APIC
+EXPORT_SYMBOL(smp_found_config);
+#endif
+#include <linux/vt_kern.h>
+EXPORT_SYMBOL(kd_mksound);
+extern void *__start_rtlinux_funcs,*__stop_rtlinux_funcs;
+EXPORT_SYMBOL(__start_rtlinux_funcs);
+EXPORT_SYMBOL(__stop_rtlinux_funcs);
+EXPORT_SYMBOL(irq_control);
+extern unsigned long (*do_gettimeoffset)(void);
+extern int use_tsc;
+EXPORT_SYMBOL(do_gettimeoffset);
+EXPORT_SYMBOL(use_tsc);
+EXPORT_SYMBOL(tick);
+extern unsigned long wall_jiffies;
+extern rwlock_t xtime_lock;
+EXPORT_SYMBOL(wall_jiffies);
+EXPORT_SYMBOL(xtime_lock);
+extern unsigned char *rtl_cached21;
+EXPORT_SYMBOL(rtl_cached21);
+extern void (*rtl_do_exit_handler)(long code);
+extern void (*rtl_syscall_intercept)(struct pt_regs regs);
+EXPORT_SYMBOL(init_tss);
+EXPORT_SYMBOL(rtl_syscall_intercept);
+EXPORT_SYMBOL(rtl_do_exit_handler);
+#ifdef CONFIG_SMP
+EXPORT_SYMBOL(flush_tlb_all);
+extern int rtl_reserved_cpumask;
+EXPORT_SYMBOL(rtl_reserved_cpumask);
+extern unsigned long irq_affinity[NR_IRQS];
+EXPORT_SYMBOL(irq_affinity);
+extern volatile int physical_apicid_2_cpu[];
+EXPORT_SYMBOL(physical_apicid_2_cpu);
+extern unsigned long flush_cpumask;
+EXPORT_SYMBOL(flush_cpumask);
+EXPORT_SYMBOL(cpu_tlbstate);
+#endif /* CONFIG_SMP */
+extern struct console *console_drivers;
+EXPORT_SYMBOL(console_drivers);
+EXPORT_SYMBOL(set_ldt_desc);
+EXPORT_SYMBOL(default_ldt);
+#endif /* CONFIG_RTLINUX */
+
 #ifdef CONFIG_MULTIQUAD
 EXPORT_SYMBOL(xquad_portio);
 #endif
diff -rbNu linux-2.4.31/arch/i386/kernel/i8259.c linux-2.4.31-rtl3.1/arch/i386/kernel/i8259.c
--- linux-2.4.31/arch/i386/kernel/i8259.c	2004-08-08 01:26:04.000000000 +0200
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/i8259.c	2008-01-17 09:18:44.929924500 +0100
@@ -132,6 +132,14 @@
 
 spinlock_t i8259A_lock = SPIN_LOCK_UNLOCKED;
 
+#ifdef CONFIG_RTLINUX
+#undef spin_lock_irqsave
+#define spin_lock_irqsave(lock, flags)		do { rtl_hard_local_irq_save_kernel(flags);       spin_lock(lock); } while (0)
+
+#undef spin_unlock_irqrestore
+#define spin_unlock_irqrestore(lock, flags)	do { spin_unlock(lock);  rtl_hard_local_irq_restore_kernel(flags); } while (0)
+#endif
+
 static void end_8259A_irq (unsigned int irq)
 {
 	if (!(irq_desc[irq].status & (IRQ_DISABLED|IRQ_INPROGRESS)))
@@ -172,6 +180,8 @@
 #define cached_21	(__byte(0,cached_irq_mask))
 #define cached_A1	(__byte(1,cached_irq_mask))
 
+unsigned char *rtl_cached21=&cached_21;
+
 /*
  * Not all IRQs can be routed through the IO-APIC, eg. on certain (older)
  * boards the timer interrupt is not really connected to any IO-APIC pin,
@@ -344,20 +354,20 @@
 	outb(0xff, 0xA1);	/* mask all of 8259A-2 */
 
 	/*
-	 * outb_p - this has to work on a wide range of PC hardware.
+	 * outb - this has to work on a wide range of PC hardware.
 	 */
-	outb_p(0x11, 0x20);	/* ICW1: select 8259A-1 init */
-	outb_p(0x20 + 0, 0x21);	/* ICW2: 8259A-1 IR0-7 mapped to 0x20-0x27 */
-	outb_p(0x04, 0x21);	/* 8259A-1 (the master) has a slave on IR2 */
+	outb(0x11, 0x20);	/* ICW1: select 8259A-1 init */
+	outb(0x20 + 0, 0x21);	/* ICW2: 8259A-1 IR0-7 mapped to 0x20-0x27 */
+	outb(0x04, 0x21);	/* 8259A-1 (the master) has a slave on IR2 */
 	if (auto_eoi)
-		outb_p(0x03, 0x21);	/* master does Auto EOI */
+		outb(0x03, 0x21);	/* master does Auto EOI */
 	else
-		outb_p(0x01, 0x21);	/* master expects normal EOI */
+		outb(0x01, 0x21);	/* master expects normal EOI */
 
-	outb_p(0x11, 0xA0);	/* ICW1: select 8259A-2 init */
-	outb_p(0x20 + 8, 0xA1);	/* ICW2: 8259A-2 IR0-7 mapped to 0x28-0x2f */
-	outb_p(0x02, 0xA1);	/* 8259A-2 is a slave on master's IR2 */
-	outb_p(0x01, 0xA1);	/* (slave's support for AEOI in flat mode
+	outb(0x11, 0xA0);	/* ICW1: select 8259A-2 init */
+	outb(0x20 + 8, 0xA1);	/* ICW2: 8259A-2 IR0-7 mapped to 0x28-0x2f */
+	outb(0x02, 0xA1);	/* 8259A-2 is a slave on master's IR2 */
+	outb(0x01, 0xA1);	/* (slave's support for AEOI in flat mode
 				    is to be investigated) */
 
 	if (auto_eoi)
@@ -494,8 +504,8 @@
 	 * Set the clock to HZ Hz, we already have a valid
 	 * vector now:
 	 */
-	outb_p(0x34,0x43);		/* binary, mode 2, LSB/MSB, ch 0 */
-	outb_p(LATCH & 0xff , 0x40);	/* LSB */
+	outb(0x34,0x43);		/* binary, mode 2, LSB/MSB, ch 0 */
+	outb(LATCH & 0xff , 0x40);	/* LSB */
 	outb(LATCH >> 8 , 0x40);	/* MSB */
 
 #ifndef CONFIG_VISWS
diff -rbNu linux-2.4.31/arch/i386/kernel/io_apic.c linux-2.4.31-rtl3.1/arch/i386/kernel/io_apic.c
--- linux-2.4.31/arch/i386/kernel/io_apic.c	2004-11-17 12:54:21.000000000 +0100
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/io_apic.c	2008-01-17 09:18:44.945510500 +0100
@@ -46,6 +46,14 @@
 unsigned char int_delivery_mode = dest_LowestPrio;
 
 
+#ifdef CONFIG_RTLINUX
+#undef spin_lock_irqsave
+#define spin_lock_irqsave(lock, flags)		do { rtl_hard_local_irq_save_kernel(flags);       spin_lock(lock); } while (0)
+
+#undef spin_unlock_irqrestore
+#define spin_unlock_irqrestore(lock, flags)	do { spin_unlock(lock);  rtl_hard_local_irq_restore_kernel(flags); } while (0)
+#endif
+
 /*
  * # of IRQ routing registers
  */
diff -rbNu linux-2.4.31/arch/i386/kernel/irq.c linux-2.4.31-rtl3.1/arch/i386/kernel/irq.c
--- linux-2.4.31/arch/i386/kernel/irq.c	2003-11-28 19:26:19.000000000 +0100
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/irq.c	2008-01-17 09:18:44.961096500 +0100
@@ -638,12 +638,19 @@
 		spin_unlock(&desc->lock);
 		handle_IRQ_event(irq, &regs, action);
 		spin_lock(&desc->lock);
+#ifdef CONFIG_IRQ_TLBFLUSH
+		__asm__ __volatile__("invlpg %0": :"m" (*(char *)&action));
+#endif 
 		
 		if (!(desc->status & IRQ_PENDING))
 			break;
 		desc->status &= ~IRQ_PENDING;
 	}
 	desc->status &= ~IRQ_INPROGRESS;
+
+#ifdef CONFIG_IRQ_TLBFLUSH
+	__asm__ __volatile__("invlpg %0": :"m" (*(char *)current));
+#endif 
 out:
 	/*
 	 * The ->end() handler has to deal with interrupts which got
@@ -652,8 +659,12 @@
 	desc->handler->end(irq);
 	spin_unlock(&desc->lock);
 
-	if (softirq_pending(cpu))
+	if (softirq_pending(cpu)){
 		do_softirq();
+#ifdef CONFIG_IRQ_TLBFLUSH
+		__asm__ __volatile__("invlpg %0": :"m" (*(char *)&do_softirq));
+#endif 
+	}
 	return 1;
 }
 
@@ -1091,7 +1102,7 @@
 
 static struct proc_dir_entry * smp_affinity_entry [NR_IRQS];
 
-static unsigned long irq_affinity [NR_IRQS] = { [0 ... NR_IRQS-1] = ~0UL };
+unsigned long irq_affinity [NR_IRQS] = { [0 ... NR_IRQS-1] = ~0UL };
 static int irq_affinity_read_proc (char *page, char **start, off_t off,
 			int count, int *eof, void *data)
 {
diff -rbNu linux-2.4.31/arch/i386/kernel/nmi.c linux-2.4.31-rtl3.1/arch/i386/kernel/nmi.c
--- linux-2.4.31/arch/i386/kernel/nmi.c	2003-06-13 16:51:29.000000000 +0200
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/nmi.c	2008-01-17 09:18:44.976682500 +0100
@@ -344,7 +344,12 @@
 	 * Since current-> is always on the stack, and we always switch
 	 * the stack NMI-atomically, it's safe to use smp_processor_id().
 	 */
+#ifdef CONFIG_RTLINUX
+	/* For RTLinux this is not always the case, hence hard_smp_processor_id */
+	int sum, cpu = hw_smp_processor_id();
+#else
 	int sum, cpu = smp_processor_id();
+#endif
 
 	sum = apic_timer_irqs[cpu];
 
diff -rbNu linux-2.4.31/arch/i386/kernel/pci-irq.c linux-2.4.31-rtl3.1/arch/i386/kernel/pci-irq.c
--- linux-2.4.31/arch/i386/kernel/pci-irq.c	2005-06-01 02:56:56.000000000 +0200
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/pci-irq.c	2008-01-17 09:18:44.992268500 +0100
@@ -573,14 +573,14 @@
  */
 static int pirq_serverworks_get(struct pci_dev *router, struct pci_dev *dev, int pirq)
 {
-	outb_p(pirq, 0xc00);
+	outb(pirq, 0xc00);
 	return inb(0xc01) & 0xf;
 }
 
 static int pirq_serverworks_set(struct pci_dev *router, struct pci_dev *dev, int pirq, int irq)
 {
-	outb_p(pirq, 0xc00);
-	outb_p(irq, 0xc01);
+	outb(pirq, 0xc00);
+	outb(irq, 0xc01);
 	return 1;
 }
 
diff -rbNu linux-2.4.31/arch/i386/kernel/process.c linux-2.4.31-rtl3.1/arch/i386/kernel/process.c
--- linux-2.4.31/arch/i386/kernel/process.c	2004-11-17 12:54:21.000000000 +0100
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/process.c	2008-01-17 09:18:45.007854500 +0100
@@ -269,7 +269,7 @@
 	int i;
 
 	for (i=0; i<0x10000; i++)
-		if ((inb_p(0x64) & 0x02) == 0)
+		if ((inb(0x64) & 0x02) == 0)
 			break;
 }
 
diff -rbNu linux-2.4.31/arch/i386/kernel/rtlinux.c linux-2.4.31-rtl3.1/arch/i386/kernel/rtlinux.c
--- linux-2.4.31/arch/i386/kernel/rtlinux.c	1970-01-01 01:00:00.000000000 +0100
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/rtlinux.c	2008-01-17 09:18:45.007854500 +0100
@@ -0,0 +1,80 @@
+#include <linux/config.h>
+#include <linux/ptrace.h>
+#include <linux/errno.h>
+#include <linux/signal.h>
+#include <linux/sched.h>
+#include <linux/ioport.h>
+#include <linux/interrupt.h>
+#include <linux/timex.h>
+#include <linux/malloc.h>
+#include <linux/random.h>
+#include <linux/smp_lock.h>
+#include <linux/init.h>
+#include <linux/kernel_stat.h>
+
+#include <asm/system.h>
+#include <asm/io.h>
+#include <asm/irq.h>
+#include <asm/bitops.h>
+#include <asm/pgtable.h>
+#include <asm/delay.h>
+#include <asm/desc.h>
+
+#include <linux/irq.h>
+
+static void rtl_hard_save_flags_f(unsigned long  *z){
+	unsigned long y;
+	rtl_hard_save_flags_kernel(y);
+	*z=y;
+}
+static void rtl_hard_restore_flags_f(unsigned long  x) { rtl_hard_restore_flags_kernel(x); }
+static void rtl_hard_cli_f(void) { rtl_hard_cli_kernel(); }
+static void rtl_hard_sti_f(void) { rtl_hard_sti_kernel(); }
+static void rtl_hard_local_irq_save_f(unsigned long  *x)
+{ unsigned long y;rtl_hard_local_irq_save_kernel(y);*x=y; }
+static void rtl_hard_local_irq_restore_f(unsigned long  x){
+	rtl_hard_local_irq_restore_kernel(x);
+}
+struct irq_control_s irq_control = {
+ 	rtl_hard_save_flags_f,
+ 	rtl_hard_restore_flags_f,
+ 	rtl_hard_cli_f,
+ 	rtl_hard_sti_f,
+ 	rtl_hard_local_irq_save_f,
+ 	rtl_hard_local_irq_restore_f
+ };
+void *rtl_emulate_iret = 0;
+void *rtl_exception_intercept = 0;
+void *rtl_syscall_intercept = 0;
+
+extern void *__start_rtlinux_patch,*__stop_rtlinux_patch;
+extern asmlinkage void do_IRQ(struct pt_regs );
+extern void ret_from_intr(void);
+RTLINUX_EXPORT(__start_rtlinux_patch);
+RTLINUX_EXPORT(__stop_rtlinux_patch);
+RTLINUX_EXPORT(rtl_emulate_iret);
+RTLINUX_EXPORT(irq_desc);
+RTLINUX_EXPORT(do_IRQ);
+RTLINUX_EXPORT(ret_from_intr);
+RTLINUX_EXPORT(rtl_exception_intercept);
+
+#ifdef CONFIG_X86_LOCAL_APIC
+extern asmlinkage void smp_spurious_interrupt(void);
+extern asmlinkage void smp_error_interrupt(void);
+extern void smp_apic_timer_interrupt(struct pt_regs *);
+
+RTLINUX_EXPORT(smp_spurious_interrupt);
+RTLINUX_EXPORT(smp_error_interrupt);
+RTLINUX_EXPORT(smp_apic_timer_interrupt);
+#endif
+
+#ifdef CONFIG_SMP
+extern void rtl_reschedule(int cpu);
+extern asmlinkage void smp_reschedule_interrupt(void);
+extern asmlinkage void smp_invalidate_interrupt(void);
+extern asmlinkage void smp_call_function_interrupt(void);
+RTLINUX_EXPORT(smp_reschedule_interrupt);
+RTLINUX_EXPORT(smp_invalidate_interrupt);
+RTLINUX_EXPORT(smp_call_function_interrupt);
+RTLINUX_EXPORT(rtl_reschedule);
+#endif
diff -rbNu linux-2.4.31/arch/i386/kernel/setup.c linux-2.4.31-rtl3.1/arch/i386/kernel/setup.c
--- linux-2.4.31/arch/i386/kernel/setup.c	2005-04-04 03:42:19.000000000 +0200
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/setup.c	2008-01-17 09:18:45.023440500 +0100
@@ -272,7 +272,7 @@
 {
 	int raw;
 
-	visws_board_type = (char)(inb_p(PIIX_GPI_BD_REG) & PIIX_GPI_BD_REG)
+	visws_board_type = (char)(inb(PIIX_GPI_BD_REG) & PIIX_GPI_BD_REG)
 							 >> PIIX_GPI_BD_SHIFT;
 /*
  * Get Board rev.
@@ -280,40 +280,40 @@
  * to the GPIO registers.  Let's map them at 0x0fc0 which is right
  * after the PIIX4 PM section.
  */
-	outb_p(SIO_DEV_SEL, SIO_INDEX);
-	outb_p(SIO_GP_DEV, SIO_DATA);	/* Talk to GPIO regs. */
+	outb(SIO_DEV_SEL, SIO_INDEX);
+	outb(SIO_GP_DEV, SIO_DATA);	/* Talk to GPIO regs. */
     
-	outb_p(SIO_DEV_MSB, SIO_INDEX);
-	outb_p(SIO_GP_MSB, SIO_DATA);	/* MSB of GPIO base address */
+	outb(SIO_DEV_MSB, SIO_INDEX);
+	outb(SIO_GP_MSB, SIO_DATA);	/* MSB of GPIO base address */
 
-	outb_p(SIO_DEV_LSB, SIO_INDEX);
-	outb_p(SIO_GP_LSB, SIO_DATA);	/* LSB of GPIO base address */
+	outb(SIO_DEV_LSB, SIO_INDEX);
+	outb(SIO_GP_LSB, SIO_DATA);	/* LSB of GPIO base address */
 
-	outb_p(SIO_DEV_ENB, SIO_INDEX);
-	outb_p(1, SIO_DATA);		/* Enable GPIO registers. */
+	outb(SIO_DEV_ENB, SIO_INDEX);
+	outb(1, SIO_DATA);		/* Enable GPIO registers. */
     
 /*
  * Now, we have to map the power management section to write
  * a bit which enables access to the GPIO registers.
  * What lunatic came up with this shit?
  */
-	outb_p(SIO_DEV_SEL, SIO_INDEX);
-	outb_p(SIO_PM_DEV, SIO_DATA);	/* Talk to GPIO regs. */
+	outb(SIO_DEV_SEL, SIO_INDEX);
+	outb(SIO_PM_DEV, SIO_DATA);	/* Talk to GPIO regs. */
 
-	outb_p(SIO_DEV_MSB, SIO_INDEX);
-	outb_p(SIO_PM_MSB, SIO_DATA);	/* MSB of PM base address */
+	outb(SIO_DEV_MSB, SIO_INDEX);
+	outb(SIO_PM_MSB, SIO_DATA);	/* MSB of PM base address */
     
-	outb_p(SIO_DEV_LSB, SIO_INDEX);
-	outb_p(SIO_PM_LSB, SIO_DATA);	/* LSB of PM base address */
+	outb(SIO_DEV_LSB, SIO_INDEX);
+	outb(SIO_PM_LSB, SIO_DATA);	/* LSB of PM base address */
 
-	outb_p(SIO_DEV_ENB, SIO_INDEX);
-	outb_p(1, SIO_DATA);		/* Enable PM registers. */
+	outb(SIO_DEV_ENB, SIO_INDEX);
+	outb(1, SIO_DATA);		/* Enable PM registers. */
     
 /*
  * Now, write the PM register which enables the GPIO registers.
  */
-	outb_p(SIO_PM_FER2, SIO_PM_INDEX);
-	outb_p(SIO_PM_GP_EN, SIO_PM_DATA);
+	outb(SIO_PM_FER2, SIO_PM_INDEX);
+	outb(SIO_PM_GP_EN, SIO_PM_DATA);
     
 /*
  * Now, initialize the GPIO registers.
@@ -321,7 +321,7 @@
  * power on default, so let's leave them alone.
  * So, let's just read the board rev!
  */
-	raw = inb_p(SIO_GP_DATA1);
+	raw = inb(SIO_GP_DATA1);
 	raw &= 0x7f;	/* 7 bits of valid board revision ID. */
 
 	if (visws_board_type == VISWS_320) {
diff -rbNu linux-2.4.31/arch/i386/kernel/smp.c linux-2.4.31-rtl3.1/arch/i386/kernel/smp.c
--- linux-2.4.31/arch/i386/kernel/smp.c	2004-11-17 12:54:21.000000000 +0100
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/smp.c	2008-01-17 09:18:45.054612500 +0100
@@ -134,6 +134,10 @@
 	 */
 	unsigned int cfg;
 
+#ifdef CONFIG_RTLINUX
+	unsigned long flags;
+	rtl_hard_local_irq_save_kernel(flags);
+#endif
 	/*
 	 * Wait for idle.
 	 */
@@ -148,6 +152,9 @@
 	 * Send the IPI. The write to APIC_ICR fires this off.
 	 */
 	apic_write_around(APIC_ICR, cfg);
+#ifdef CONFIG_RTLINUX
+	rtl_hard_local_irq_restore_kernel(flags);
+#endif
 }
 
 void fastcall send_IPI_self(int vector)
@@ -160,8 +167,12 @@
 	unsigned long cfg;
 	unsigned long flags;
 
+#ifdef CONFIG_RTLINUX
+	rtl_hard_local_irq_save_kernel(flags);
+#else
 	__save_flags(flags);
 	__cli();
+#endif
 
 		
 	/*
@@ -185,7 +196,11 @@
 	 */
 	apic_write_around(APIC_ICR, cfg);
 
+#ifdef CONFIG_RTLINUX
+	rtl_hard_local_irq_restore_kernel(flags);
+#else
 	__restore_flags(flags);
+#endif
 }
 
 static inline void send_IPI_mask_sequence(int mask, int vector)
@@ -199,8 +214,12 @@
 	 * should be modified to do 1 message per cluster ID - mbligh
 	 */ 
 
+#ifdef CONFIG_RTLINUX
+	rtl_hard_local_irq_save_kernel(flags);
+#else
 	__save_flags(flags);
 	__cli();
+#endif
 
 	for (query_cpu = 0; query_cpu < NR_CPUS; ++query_cpu) {
 		query_mask = 1 << query_cpu;
@@ -231,7 +250,11 @@
 			apic_write_around(APIC_ICR, cfg);
 		}
 	}
+#ifdef CONFIG_RTLINUX
+	rtl_hard_local_irq_restore_kernel(flags);
+#else
 	__restore_flags(flags);
+#endif
 }
 
 static inline void send_IPI_mask(int mask, int vector)
@@ -292,7 +315,7 @@
  *	Optimizations Manfred Spraul <manfred@colorfullife.com>
  */
 
-static volatile unsigned long flush_cpumask;
+volatile unsigned long flush_cpumask;
 static struct mm_struct * flush_mm;
 static unsigned long flush_va;
 static spinlock_t tlbstate_lock = SPIN_LOCK_UNLOCKED;
@@ -386,6 +409,10 @@
 	clear_bit(cpu, &flush_cpumask);
 }
 
+#ifdef CONFIG_RTLINUX
+int rtl_reserved_cpumask;
+#endif
+
 static void flush_tlb_others (unsigned long cpumask, struct mm_struct *mm,
 						unsigned long va)
 {
@@ -416,6 +443,10 @@
 	flush_mm = mm;
 	flush_va = va;
 	atomic_set_mask(cpumask, &flush_cpumask);
+#ifdef CONFIG_RTLINUX
+	/* make shure that reserved cpu's are excluded ! */
+	atomic_clear_mask(rtl_reserved_cpumask,&flush_cpumask);
+#endif
 	/*
 	 * We have to send the IPI only to
 	 * CPUs affected.
@@ -502,6 +533,13 @@
 	send_IPI_mask(1 << cpu, RESCHEDULE_VECTOR);
 }
 
+#ifdef CONFIG_RTLINUX
+#define RTL_RESCHEDULE_VECTOR 0xEE
+void rtl_reschedule(int cpu)
+{
+	send_IPI_mask(1 << cpu, RTL_RESCHEDULE_VECTOR);
+}
+#endif
 /*
  * Structure and data for smp_call_function(). This is designed to minimise
  * static memory requirements. It also looks cleaner.
diff -rbNu linux-2.4.31/arch/i386/kernel/time.c linux-2.4.31-rtl3.1/arch/i386/kernel/time.c
--- linux-2.4.31/arch/i386/kernel/time.c	2004-02-18 14:36:30.000000000 +0100
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/time.c	2008-01-17 09:18:45.054612500 +0100
@@ -121,7 +121,7 @@
 
 extern spinlock_t i8259A_lock;
 
-#ifndef CONFIG_X86_TSC
+#if defined (CONFIG_RTLINUX) || ! defined (CONFIG_X86_TSC)
 
 /* This function must be called with interrupts disabled 
  * It was inspired by Steve McCanne's microtime-i386 for BSD.  -- jrs
@@ -170,9 +170,9 @@
 	/* gets recalled with irq locally disabled */
 	spin_lock(&i8253_lock);
 	/* timer count may underflow right here */
-	outb_p(0x00, 0x43);	/* latch the count ASAP */
+	outb(0x00, 0x43);	/* latch the count ASAP */
 
-	count = inb_p(0x40);	/* read the latched count */
+	count = inb(0x40);	/* read the latched count */
 
 	/*
 	 * We do this guaranteed double memory access instead of a _p 
@@ -180,12 +180,12 @@
 	 */
  	jiffies_t = jiffies;
 
-	count |= inb_p(0x40) << 8;
+	count |= inb(0x40) << 8;
 	
         /* VIA686a test code... reset the latch if count > max + 1 */
         if (count > LATCH) {
-                outb_p(0x34, 0x43);
-                outb_p(LATCH & 0xff, 0x40);
+                outb(0x34, 0x43);
+                outb(LATCH & 0xff, 0x40);
                 outb(LATCH >> 8, 0x40);
                 count = LATCH - 1;
         }
@@ -211,6 +211,10 @@
 
 			int i;
 
+#ifdef CONFIG_RTLINUX
+			unsigned long flags;
+			rtl_hard_local_irq_save_kernel(flags);
+#endif
 			spin_lock(&i8259A_lock);
 			/*
 			 * This is tricky when I/O APICs are used;
@@ -218,6 +222,9 @@
 			 */
 			i = inb(0x20);
 			spin_unlock(&i8259A_lock);
+#ifdef CONFIG_RTLINUX
+			rtl_hard_local_irq_restore_kernel(flags);
+#endif
 
 			/* assumption about timer being IRQ0 */
 			if (i & 0x01) {
@@ -254,7 +261,7 @@
 	return count;
 }
 
-static unsigned long (*do_gettimeoffset)(void) = do_slow_gettimeoffset;
+unsigned long (*do_gettimeoffset)(void) = do_slow_gettimeoffset;
 
 
 /* IBM Summit (EXA) Cyclone Timer code*/
@@ -577,11 +584,18 @@
 		 * This will also deassert NMI lines for the watchdog if run
 		 * on an 82489DX-based system.
 		 */
+#ifdef CONFIG_RTLINUX
+		unsigned long flags;
+		rtl_hard_local_irq_save_kernel(flags);
+#endif
 		spin_lock(&i8259A_lock);
 		outb(0x0c, 0x20);
 		/* Ack the IRQ; AEOI will end it automatically. */
 		inb(0x20);
 		spin_unlock(&i8259A_lock);
+#ifdef CONFIG_RTLINUX
+		rtl_hard_local_irq_restore_kernel(flags);
+#endif
 	}
 #endif
 
@@ -629,13 +643,13 @@
 		high bit of the PPI port B (0x61).  Note that some PS/2s,
 		notably the 55SX, work fine if this is removed.  */
 
-		irq = inb_p( 0x61 );	/* read the current state */
-		outb_p( irq|0x80, 0x61 );	/* reset the IRQ */
+		irq = inb( 0x61 );	/* read the current state */
+		outb( irq|0x80, 0x61 );	/* reset the IRQ */
 	}
 #endif
 }
 
-static int use_tsc;
+int use_tsc;
 
 /*
  * This is the same as the above, except we _also_ save the current
@@ -675,9 +689,9 @@
 		rdtscl(last_tsc_low);
 
 		spin_lock(&i8253_lock);
-		outb_p(0x00, 0x43);     /* latch the count ASAP */
+		outb(0x00, 0x43);     /* latch the count ASAP */
 
-		count = inb_p(0x40);    /* read the latched count */
+		count = inb(0x40);    /* read the latched count */
 		count |= inb(0x40) << 8;
 
 		/* Any unpaired read will cause the above to swap MSB/LSB
diff -rbNu linux-2.4.31/arch/i386/kernel/traps.c linux-2.4.31-rtl3.1/arch/i386/kernel/traps.c
--- linux-2.4.31/arch/i386/kernel/traps.c	2002-11-29 00:53:09.000000000 +0100
+++ linux-2.4.31-rtl3.1/arch/i386/kernel/traps.c	2008-01-17 09:18:45.070198500 +0100
@@ -467,11 +467,16 @@
 	printk("Do you have a strange power saving mode enabled?\n");
 }
 
+
 asmlinkage void do_nmi(struct pt_regs * regs, long error_code)
 {
 	unsigned char reason = inb(0x61);
 
+#ifdef CONFIG_RTLINUX
+	++nmi_count(hw_smp_processor_id());
+#else
 	++nmi_count(smp_processor_id());
+#endif
 
 	if (!(reason & 0xc0)) {
 #if CONFIG_X86_LOCAL_APIC
diff -rbNu linux-2.4.31/arch/i386/mm/fault.c linux-2.4.31-rtl3.1/arch/i386/mm/fault.c
--- linux-2.4.31/arch/i386/mm/fault.c	2004-08-08 01:26:04.000000000 +0200
+++ linux-2.4.31-rtl3.1/arch/i386/mm/fault.c	2008-01-17 09:18:45.085784500 +0100
@@ -25,6 +25,9 @@
 #include <asm/pgalloc.h>
 #include <asm/hardirq.h>
 
+// need that for hard_sti in do_page_fault - thanks to Paolo Mantegazza .
+#include <asm/rtlinux_cli.h>
+
 extern void die(const char *,struct pt_regs *,long);
 
 /*
@@ -153,8 +156,8 @@
 
 	/* It's safe to allow irq's after cr2 has been saved */
 	if (regs->eflags & X86_EFLAGS_IF)
-		local_irq_enable();
-
+//		local_irq_enable();
+ 		rtl_hard_sti_kernel();
 	tsk = current;
 
 	/*
diff -rbNu linux-2.4.31/arch/i386/mm/ioremap.c linux-2.4.31-rtl3.1/arch/i386/mm/ioremap.c
--- linux-2.4.31/arch/i386/mm/ioremap.c	2003-11-28 19:26:19.000000000 +0100
+++ linux-2.4.31-rtl3.1/arch/i386/mm/ioremap.c	2008-01-17 09:18:45.101370500 +0100
@@ -82,6 +82,7 @@
 					 phys_addr + address, flags))
 			break;
 		error = 0;
+		set_pgdir(address, *dir);
 		address = (address + PGDIR_SIZE) & PGDIR_MASK;
 		dir++;
 	} while (address && (address < end));
diff -rbNu linux-2.4.31/config_rtl linux-2.4.31-rtl3.1/config_rtl
--- linux-2.4.31/config_rtl	1970-01-01 01:00:00.000000000 +0100
+++ linux-2.4.31-rtl3.1/config_rtl	2008-01-17 09:18:45.116956500 +0100
@@ -0,0 +1,1009 @@
+#
+# Automatically generated make config: don't edit
+#
+CONFIG_X86=y
+# CONFIG_SBUS is not set
+
+#
+# Enable RTLinux HAL
+#
+CONFIG_RTLINUX=y
+CONFIG_RTL=y
+CONFIG_IRQ_TLBFLUSH=y
+CONFIG_NET_TLBFLUSH=y
+CONFIG_UID16=y
+
+#
+# Code maturity level options
+#
+CONFIG_EXPERIMENTAL=y
+
+#
+# Loadable module support
+#
+CONFIG_MODULES=y
+# CONFIG_MODVERSIONS is not set
+CONFIG_KMOD=y
+
+#
+# Processor type and features
+#
+# CONFIG_M386 is not set
+# CONFIG_M486 is not set
+CONFIG_M586=y
+# CONFIG_M586TSC is not set
+# CONFIG_M586MMX is not set
+# CONFIG_M686 is not set
+# CONFIG_MPENTIUMIII is not set
+# CONFIG_MPENTIUM4 is not set
+# CONFIG_MK6 is not set
+# CONFIG_MK7 is not set
+# CONFIG_MK8 is not set
+# CONFIG_MELAN is not set
+# CONFIG_MCRUSOE is not set
+# CONFIG_MWINCHIPC6 is not set
+# CONFIG_MWINCHIP2 is not set
+# CONFIG_MWINCHIP3D is not set
+# CONFIG_MCYRIXIII is not set
+# CONFIG_MVIAC3_2 is not set
+CONFIG_X86_WP_WORKS_OK=y
+CONFIG_X86_INVLPG=y
+CONFIG_X86_CMPXCHG=y
+CONFIG_X86_XADD=y
+CONFIG_X86_BSWAP=y
+CONFIG_X86_POPAD_OK=y
+# CONFIG_RWSEM_GENERIC_SPINLOCK is not set
+CONFIG_RWSEM_XCHGADD_ALGORITHM=y
+CONFIG_X86_L1_CACHE_SHIFT=5
+CONFIG_X86_USE_STRING_486=y
+CONFIG_X86_ALIGNMENT_16=y
+CONFIG_X86_PPRO_FENCE=y
+# CONFIG_X86_F00F_WORKS_OK is not set
+# CONFIG_X86_MCE is not set
+# CONFIG_TOSHIBA is not set
+# CONFIG_I8K is not set
+# CONFIG_MICROCODE is not set
+# CONFIG_X86_MSR is not set
+# CONFIG_X86_CPUID is not set
+# CONFIG_EDD is not set
+CONFIG_NOHIGHMEM=y
+# CONFIG_HIGHMEM4G is not set
+# CONFIG_HIGHMEM64G is not set
+# CONFIG_HIGHMEM is not set
+# CONFIG_MATH_EMULATION is not set
+# CONFIG_MTRR is not set
+# CONFIG_SMP is not set
+# CONFIG_X86_UP_APIC is not set
+# CONFIG_X86_UP_IOAPIC is not set
+# CONFIG_X86_TSC_DISABLE is not set
+
+#
+# General setup
+#
+CONFIG_NET=y
+CONFIG_PCI=y
+# CONFIG_PCI_GOBIOS is not set
+# CONFIG_PCI_GODIRECT is not set
+CONFIG_PCI_GOANY=y
+CONFIG_PCI_BIOS=y
+CONFIG_PCI_DIRECT=y
+# CONFIG_ISA is not set
+CONFIG_PCI_NAMES=y
+# CONFIG_EISA is not set
+# CONFIG_MCA is not set
+# CONFIG_HOTPLUG is not set
+# CONFIG_PCMCIA is not set
+# CONFIG_HOTPLUG_PCI is not set
+CONFIG_SYSVIPC=y
+CONFIG_BSD_PROCESS_ACCT=y
+CONFIG_SYSCTL=y
+CONFIG_KCORE_ELF=y
+# CONFIG_KCORE_AOUT is not set
+CONFIG_BINFMT_AOUT=m
+CONFIG_BINFMT_ELF=y
+# CONFIG_BINFMT_MISC is not set
+# CONFIG_OOM_KILLER is not set
+# CONFIG_PM is not set
+# CONFIG_APM is not set
+
+#
+# ACPI Support
+#
+# CONFIG_ACPI is not set
+
+#
+# Memory Technology Devices (MTD)
+#
+# CONFIG_MTD is not set
+
+#
+# Parallel port support
+#
+# CONFIG_PARPORT is not set
+
+#
+# Plug and Play configuration
+#
+# CONFIG_PNP is not set
+# CONFIG_ISAPNP is not set
+
+#
+# Block devices
+#
+CONFIG_BLK_DEV_FD=y
+# CONFIG_BLK_DEV_XD is not set
+# CONFIG_PARIDE is not set
+# CONFIG_BLK_CPQ_DA is not set
+# CONFIG_BLK_CPQ_CISS_DA is not set
+# CONFIG_CISS_SCSI_TAPE is not set
+# CONFIG_CISS_MONITOR_THREAD is not set
+# CONFIG_BLK_DEV_DAC960 is not set
+# CONFIG_BLK_DEV_UMEM is not set
+# CONFIG_BLK_DEV_SX8 is not set
+CONFIG_BLK_DEV_LOOP=y
+CONFIG_BLK_DEV_NBD=m
+CONFIG_BLK_DEV_RAM=y
+CONFIG_BLK_DEV_RAM_SIZE=32768
+CONFIG_BLK_DEV_INITRD=y
+# CONFIG_BLK_STATS is not set
+
+#
+# Multi-device support (RAID and LVM)
+#
+# CONFIG_MD is not set
+# CONFIG_BLK_DEV_MD is not set
+# CONFIG_MD_LINEAR is not set
+# CONFIG_MD_RAID0 is not set
+# CONFIG_MD_RAID1 is not set
+# CONFIG_MD_RAID5 is not set
+# CONFIG_MD_MULTIPATH is not set
+# CONFIG_BLK_DEV_LVM is not set
+
+#
+# Networking options
+#
+CONFIG_PACKET=y
+CONFIG_PACKET_MMAP=y
+CONFIG_NETLINK_DEV=m
+CONFIG_NETFILTER=y
+# CONFIG_NETFILTER_DEBUG is not set
+CONFIG_FILTER=y
+CONFIG_UNIX=y
+CONFIG_INET=y
+CONFIG_IP_MULTICAST=y
+CONFIG_IP_ADVANCED_ROUTER=y
+CONFIG_IP_MULTIPLE_TABLES=y
+CONFIG_IP_ROUTE_FWMARK=y
+CONFIG_IP_ROUTE_NAT=y
+CONFIG_IP_ROUTE_MULTIPATH=y
+CONFIG_IP_ROUTE_TOS=y
+CONFIG_IP_ROUTE_VERBOSE=y
+# CONFIG_IP_PNP is not set
+CONFIG_NET_IPIP=m
+CONFIG_NET_IPGRE=m
+CONFIG_NET_IPGRE_BROADCAST=y
+CONFIG_IP_MROUTE=y
+CONFIG_IP_PIMSM_V1=y
+# CONFIG_IP_PIMSM_V2 is not set
+# CONFIG_ARPD is not set
+# CONFIG_INET_ECN is not set
+CONFIG_SYN_COOKIES=y
+
+#
+#   IP: Netfilter Configuration
+#
+CONFIG_IP_NF_CONNTRACK=m
+CONFIG_IP_NF_FTP=m
+CONFIG_IP_NF_AMANDA=m
+CONFIG_IP_NF_TFTP=m
+CONFIG_IP_NF_IRC=m
+CONFIG_IP_NF_QUEUE=m
+CONFIG_IP_NF_IPTABLES=m
+CONFIG_IP_NF_MATCH_LIMIT=m
+CONFIG_IP_NF_MATCH_MAC=m
+CONFIG_IP_NF_MATCH_PKTTYPE=m
+CONFIG_IP_NF_MATCH_MARK=m
+CONFIG_IP_NF_MATCH_MULTIPORT=m
+CONFIG_IP_NF_MATCH_TOS=m
+CONFIG_IP_NF_MATCH_RECENT=m
+CONFIG_IP_NF_MATCH_ECN=m
+CONFIG_IP_NF_MATCH_DSCP=m
+CONFIG_IP_NF_MATCH_AH_ESP=m
+CONFIG_IP_NF_MATCH_LENGTH=m
+CONFIG_IP_NF_MATCH_TTL=m
+CONFIG_IP_NF_MATCH_TCPMSS=m
+CONFIG_IP_NF_MATCH_HELPER=m
+CONFIG_IP_NF_MATCH_STATE=m
+CONFIG_IP_NF_MATCH_CONNTRACK=m
+CONFIG_IP_NF_MATCH_UNCLEAN=m
+CONFIG_IP_NF_MATCH_OWNER=m
+CONFIG_IP_NF_FILTER=m
+CONFIG_IP_NF_TARGET_REJECT=m
+CONFIG_IP_NF_TARGET_MIRROR=m
+CONFIG_IP_NF_NAT=m
+CONFIG_IP_NF_NAT_NEEDED=y
+CONFIG_IP_NF_TARGET_MASQUERADE=m
+CONFIG_IP_NF_TARGET_REDIRECT=m
+CONFIG_IP_NF_NAT_AMANDA=m
+CONFIG_IP_NF_NAT_SNMP_BASIC=m
+CONFIG_IP_NF_NAT_IRC=m
+CONFIG_IP_NF_NAT_FTP=m
+CONFIG_IP_NF_NAT_TFTP=m
+CONFIG_IP_NF_MANGLE=m
+CONFIG_IP_NF_TARGET_TOS=m
+CONFIG_IP_NF_TARGET_ECN=m
+CONFIG_IP_NF_TARGET_DSCP=m
+CONFIG_IP_NF_TARGET_MARK=m
+CONFIG_IP_NF_TARGET_LOG=m
+CONFIG_IP_NF_TARGET_ULOG=m
+CONFIG_IP_NF_TARGET_TCPMSS=m
+CONFIG_IP_NF_ARPTABLES=m
+CONFIG_IP_NF_ARPFILTER=m
+CONFIG_IP_NF_ARP_MANGLE=m
+CONFIG_IP_NF_COMPAT_IPCHAINS=m
+CONFIG_IP_NF_NAT_NEEDED=y
+# CONFIG_IP_NF_COMPAT_IPFWADM is not set
+
+#
+#   IP: Virtual Server Configuration
+#
+CONFIG_IP_VS=m
+# CONFIG_IP_VS_DEBUG is not set
+CONFIG_IP_VS_TAB_BITS=12
+
+#
+# IPVS scheduler
+#
+CONFIG_IP_VS_RR=m
+CONFIG_IP_VS_WRR=m
+CONFIG_IP_VS_LC=m
+CONFIG_IP_VS_WLC=m
+CONFIG_IP_VS_LBLC=m
+CONFIG_IP_VS_LBLCR=m
+CONFIG_IP_VS_DH=m
+CONFIG_IP_VS_SH=m
+CONFIG_IP_VS_SED=m
+CONFIG_IP_VS_NQ=m
+
+#
+# IPVS application helper
+#
+CONFIG_IP_VS_FTP=m
+CONFIG_IPV6=m
+
+#
+#   IPv6: Netfilter Configuration
+#
+# CONFIG_IP6_NF_QUEUE is not set
+CONFIG_IP6_NF_IPTABLES=m
+CONFIG_IP6_NF_MATCH_LIMIT=m
+CONFIG_IP6_NF_MATCH_MAC=m
+CONFIG_IP6_NF_MATCH_RT=m
+CONFIG_IP6_NF_MATCH_OPTS=m
+CONFIG_IP6_NF_MATCH_FRAG=m
+CONFIG_IP6_NF_MATCH_HL=m
+CONFIG_IP6_NF_MATCH_MULTIPORT=m
+CONFIG_IP6_NF_MATCH_OWNER=m
+CONFIG_IP6_NF_MATCH_MARK=m
+CONFIG_IP6_NF_MATCH_IPV6HEADER=m
+CONFIG_IP6_NF_MATCH_AHESP=m
+CONFIG_IP6_NF_MATCH_LENGTH=m
+CONFIG_IP6_NF_MATCH_EUI64=m
+CONFIG_IP6_NF_FILTER=m
+CONFIG_IP6_NF_TARGET_LOG=m
+CONFIG_IP6_NF_MANGLE=m
+CONFIG_IP6_NF_TARGET_MARK=m
+CONFIG_KHTTPD=m
+
+#
+#    SCTP Configuration (EXPERIMENTAL)
+#
+CONFIG_IP_SCTP=m
+# CONFIG_SCTP_DBG_MSG is not set
+# CONFIG_SCTP_DBG_OBJCNT is not set
+# CONFIG_SCTP_HMAC_NONE is not set
+# CONFIG_SCTP_HMAC_SHA1 is not set
+CONFIG_SCTP_HMAC_MD5=y
+# CONFIG_ATM is not set
+CONFIG_VLAN_8021Q=m
+
+#
+#  
+#
+CONFIG_IPX=m
+# CONFIG_IPX_INTERN is not set
+CONFIG_ATALK=m
+
+#
+# Appletalk devices
+#
+# CONFIG_DEV_APPLETALK is not set
+CONFIG_DECNET=m
+# CONFIG_DECNET_SIOCGIFCONF is not set
+# CONFIG_DECNET_ROUTER is not set
+CONFIG_BRIDGE=m
+CONFIG_X25=m
+CONFIG_LAPB=m
+# CONFIG_LLC is not set
+# CONFIG_NET_DIVERT is not set
+# CONFIG_ECONET is not set
+CONFIG_WAN_ROUTER=m
+# CONFIG_NET_FASTROUTE is not set
+# CONFIG_NET_HW_FLOWCONTROL is not set
+
+#
+# QoS and/or fair queueing
+#
+CONFIG_NET_SCHED=y
+CONFIG_NET_SCH_CBQ=m
+CONFIG_NET_SCH_HTB=m
+CONFIG_NET_SCH_CSZ=m
+CONFIG_NET_SCH_HFSC=m
+CONFIG_NET_SCH_PRIO=m
+CONFIG_NET_SCH_RED=m
+CONFIG_NET_SCH_SFQ=m
+CONFIG_NET_SCH_TEQL=m
+CONFIG_NET_SCH_TBF=m
+CONFIG_NET_SCH_GRED=m
+# CONFIG_NET_SCH_NETEM is not set
+CONFIG_NET_SCH_DSMARK=m
+CONFIG_NET_SCH_INGRESS=m
+CONFIG_NET_QOS=y
+CONFIG_NET_ESTIMATOR=y
+CONFIG_NET_CLS=y
+CONFIG_NET_CLS_TCINDEX=m
+CONFIG_NET_CLS_ROUTE4=m
+CONFIG_NET_CLS_ROUTE=y
+CONFIG_NET_CLS_FW=m
+CONFIG_NET_CLS_U32=m
+CONFIG_NET_CLS_RSVP=m
+CONFIG_NET_CLS_RSVP6=m
+CONFIG_NET_CLS_POLICE=y
+
+#
+# Network testing
+#
+CONFIG_NET_PKTGEN=m
+
+#
+# Telephony Support
+#
+# CONFIG_PHONE is not set
+# CONFIG_PHONE_IXJ is not set
+# CONFIG_PHONE_IXJ_PCMCIA is not set
+
+#
+# ATA/IDE/MFM/RLL support
+#
+CONFIG_IDE=y
+
+#
+# IDE, ATA and ATAPI Block devices
+#
+CONFIG_BLK_DEV_IDE=y
+
+#
+# Please see Documentation/ide.txt for help/info on IDE drives
+#
+# CONFIG_BLK_DEV_HD_IDE is not set
+# CONFIG_BLK_DEV_HD is not set
+# CONFIG_BLK_DEV_IDE_SATA is not set
+CONFIG_BLK_DEV_IDEDISK=y
+# CONFIG_IDEDISK_MULTI_MODE is not set
+# CONFIG_IDEDISK_STROKE is not set
+# CONFIG_BLK_DEV_IDECS is not set
+# CONFIG_BLK_DEV_DELKIN is not set
+CONFIG_BLK_DEV_IDECD=m
+CONFIG_BLK_DEV_IDETAPE=m
+# CONFIG_BLK_DEV_IDEFLOPPY is not set
+CONFIG_BLK_DEV_IDESCSI=m
+# CONFIG_IDE_TASK_IOCTL is not set
+
+#
+# IDE chipset support/bugfixes
+#
+CONFIG_BLK_DEV_CMD640=y
+# CONFIG_BLK_DEV_CMD640_ENHANCED is not set
+# CONFIG_BLK_DEV_ISAPNP is not set
+CONFIG_BLK_DEV_IDEPCI=y
+CONFIG_BLK_DEV_GENERIC=y
+CONFIG_IDEPCI_SHARE_IRQ=y
+CONFIG_BLK_DEV_IDEDMA_PCI=y
+# CONFIG_BLK_DEV_OFFBOARD is not set
+# CONFIG_BLK_DEV_IDEDMA_FORCED is not set
+CONFIG_IDEDMA_PCI_AUTO=y
+# CONFIG_IDEDMA_ONLYDISK is not set
+CONFIG_BLK_DEV_IDEDMA=y
+# CONFIG_IDEDMA_PCI_WIP is not set
+CONFIG_BLK_DEV_ADMA100=y
+CONFIG_BLK_DEV_AEC62XX=y
+CONFIG_BLK_DEV_ALI15X3=y
+# CONFIG_WDC_ALI15X3 is not set
+CONFIG_BLK_DEV_AMD74XX=y
+# CONFIG_AMD74XX_OVERRIDE is not set
+CONFIG_BLK_DEV_ATIIXP=y
+CONFIG_BLK_DEV_CMD64X=y
+CONFIG_BLK_DEV_TRIFLEX=y
+# CONFIG_BLK_DEV_CY82C693 is not set
+CONFIG_BLK_DEV_CS5530=y
+CONFIG_BLK_DEV_HPT34X=y
+# CONFIG_HPT34X_AUTODMA is not set
+CONFIG_BLK_DEV_HPT366=y
+CONFIG_BLK_DEV_PIIX=y
+# CONFIG_BLK_DEV_NS87415 is not set
+# CONFIG_BLK_DEV_OPTI621 is not set
+CONFIG_BLK_DEV_PDC202XX_OLD=y
+# CONFIG_PDC202XX_BURST is not set
+CONFIG_BLK_DEV_PDC202XX_NEW=y
+# CONFIG_PDC202XX_FORCE is not set
+CONFIG_BLK_DEV_RZ1000=y
+CONFIG_BLK_DEV_SC1200=y
+CONFIG_BLK_DEV_SVWKS=y
+CONFIG_BLK_DEV_SIIMAGE=y
+CONFIG_BLK_DEV_SIS5513=y
+CONFIG_BLK_DEV_SLC90E66=y
+# CONFIG_BLK_DEV_TRM290 is not set
+CONFIG_BLK_DEV_VIA82CXXX=y
+# CONFIG_IDE_CHIPSETS is not set
+CONFIG_IDEDMA_AUTO=y
+# CONFIG_IDEDMA_IVB is not set
+# CONFIG_DMA_NONPCI is not set
+CONFIG_BLK_DEV_PDC202XX=y
+CONFIG_BLK_DEV_ATARAID=m
+CONFIG_BLK_DEV_ATARAID_PDC=m
+CONFIG_BLK_DEV_ATARAID_HPT=m
+CONFIG_BLK_DEV_ATARAID_MEDLEY=m
+CONFIG_BLK_DEV_ATARAID_SII=m
+
+#
+# SCSI support
+#
+CONFIG_SCSI=y
+
+#
+# SCSI support type (disk, tape, CD-ROM)
+#
+CONFIG_BLK_DEV_SD=y
+CONFIG_SD_EXTRA_DEVS=40
+CONFIG_CHR_DEV_ST=m
+CONFIG_CHR_DEV_OSST=m
+CONFIG_BLK_DEV_SR=m
+# CONFIG_BLK_DEV_SR_VENDOR is not set
+CONFIG_SR_EXTRA_DEVS=2
+CONFIG_CHR_DEV_SG=m
+
+#
+# Some SCSI devices (e.g. CD jukebox) support multiple LUNs
+#
+CONFIG_SCSI_DEBUG_QUEUES=y
+# CONFIG_SCSI_MULTI_LUN is not set
+# CONFIG_SCSI_CONSTANTS is not set
+# CONFIG_SCSI_LOGGING is not set
+
+#
+# SCSI low-level drivers
+#
+CONFIG_BLK_DEV_3W_XXXX_RAID=m
+CONFIG_SCSI_7000FASST=m
+CONFIG_SCSI_ACARD=m
+CONFIG_SCSI_AHA152X=m
+CONFIG_SCSI_AHA1542=m
+CONFIG_SCSI_AHA1740=m
+CONFIG_SCSI_AACRAID=m
+CONFIG_SCSI_AIC7XXX=m
+CONFIG_AIC7XXX_CMDS_PER_DEVICE=8
+CONFIG_AIC7XXX_RESET_DELAY_MS=15000
+# CONFIG_AIC7XXX_PROBE_EISA_VL is not set
+# CONFIG_AIC7XXX_BUILD_FIRMWARE is not set
+# CONFIG_AIC7XXX_DEBUG_ENABLE is not set
+CONFIG_AIC7XXX_DEBUG_MASK=0
+# CONFIG_AIC7XXX_REG_PRETTY_PRINT is not set
+CONFIG_SCSI_AIC79XX=m
+CONFIG_AIC79XX_CMDS_PER_DEVICE=32
+CONFIG_AIC79XX_RESET_DELAY_MS=15000
+# CONFIG_AIC79XX_BUILD_FIRMWARE is not set
+# CONFIG_AIC79XX_ENABLE_RD_STRM is not set
+# CONFIG_AIC79XX_DEBUG_ENABLE is not set
+CONFIG_AIC79XX_DEBUG_MASK=0
+# CONFIG_AIC79XX_REG_PRETTY_PRINT is not set
+CONFIG_SCSI_AIC7XXX_OLD=m
+# CONFIG_AIC7XXX_OLD_TCQ_ON_BY_DEFAULT is not set
+CONFIG_AIC7XXX_OLD_CMDS_PER_DEVICE=8
+# CONFIG_AIC7XXX_OLD_PROC_STATS is not set
+CONFIG_SCSI_DPT_I2O=m
+CONFIG_SCSI_ADVANSYS=m
+CONFIG_SCSI_IN2000=m
+CONFIG_SCSI_AM53C974=m
+CONFIG_SCSI_MEGARAID=m
+CONFIG_SCSI_MEGARAID2=m
+# CONFIG_SCSI_SATA is not set
+# CONFIG_SCSI_SATA_AHCI is not set
+# CONFIG_SCSI_SATA_SVW is not set
+# CONFIG_SCSI_ATA_PIIX is not set
+# CONFIG_SCSI_SATA_NV is not set
+# CONFIG_SCSI_SATA_PROMISE is not set
+# CONFIG_SCSI_SATA_SX4 is not set
+# CONFIG_SCSI_SATA_SIL is not set
+# CONFIG_SCSI_SATA_SIS is not set
+# CONFIG_SCSI_SATA_ULI is not set
+# CONFIG_SCSI_SATA_VIA is not set
+# CONFIG_SCSI_SATA_VITESSE is not set
+CONFIG_SCSI_BUSLOGIC=m
+# CONFIG_SCSI_OMIT_FLASHPOINT is not set
+CONFIG_SCSI_CPQFCTS=m
+CONFIG_SCSI_DMX3191D=m
+CONFIG_SCSI_DTC3280=m
+CONFIG_SCSI_EATA=m
+# CONFIG_SCSI_EATA_TAGGED_QUEUE is not set
+# CONFIG_SCSI_EATA_LINKED_COMMANDS is not set
+CONFIG_SCSI_EATA_MAX_TAGS=16
+CONFIG_SCSI_EATA_DMA=m
+CONFIG_SCSI_EATA_PIO=m
+CONFIG_SCSI_FUTURE_DOMAIN=m
+CONFIG_SCSI_GDTH=m
+CONFIG_SCSI_GENERIC_NCR5380=m
+CONFIG_SCSI_GENERIC_NCR53C400=y
+CONFIG_SCSI_G_NCR5380_PORT=y
+# CONFIG_SCSI_G_NCR5380_MEM is not set
+CONFIG_SCSI_IPS=m
+CONFIG_SCSI_INITIO=m
+CONFIG_SCSI_INIA100=m
+CONFIG_SCSI_NCR53C406A=m
+CONFIG_SCSI_NCR53C7xx=m
+# CONFIG_SCSI_NCR53C7xx_sync is not set
+# CONFIG_SCSI_NCR53C7xx_FAST is not set
+# CONFIG_SCSI_NCR53C7xx_DISCONNECT is not set
+CONFIG_SCSI_SYM53C8XX_2=m
+CONFIG_SCSI_SYM53C8XX_DMA_ADDRESSING_MODE=1
+CONFIG_SCSI_SYM53C8XX_DEFAULT_TAGS=16
+CONFIG_SCSI_SYM53C8XX_MAX_TAGS=64
+# CONFIG_SCSI_SYM53C8XX_IOMAPPED is not set
+CONFIG_SCSI_NCR53C8XX=m
+CONFIG_SCSI_SYM53C8XX=m
+CONFIG_SCSI_NCR53C8XX_DEFAULT_TAGS=8
+CONFIG_SCSI_NCR53C8XX_MAX_TAGS=32
+CONFIG_SCSI_NCR53C8XX_SYNC=20
+# CONFIG_SCSI_NCR53C8XX_PROFILE is not set
+# CONFIG_SCSI_NCR53C8XX_IOMAPPED is not set
+CONFIG_SCSI_NCR53C8XX_PQS_PDS=y
+# CONFIG_SCSI_NCR53C8XX_SYMBIOS_COMPAT is not set
+CONFIG_SCSI_PAS16=m
+CONFIG_SCSI_PCI2000=m
+CONFIG_SCSI_PCI2220I=m
+CONFIG_SCSI_PSI240I=m
+CONFIG_SCSI_QLOGIC_FAS=m
+CONFIG_SCSI_QLOGIC_ISP=m
+CONFIG_SCSI_QLOGIC_FC=m
+CONFIG_SCSI_QLOGIC_FC_FIRMWARE=y
+CONFIG_SCSI_QLOGIC_1280=m
+CONFIG_SCSI_SEAGATE=m
+CONFIG_SCSI_SIM710=m
+CONFIG_SCSI_SYM53C416=m
+CONFIG_SCSI_DC390T=m
+# CONFIG_SCSI_DC390T_NOGENSUPP is not set
+CONFIG_SCSI_T128=m
+CONFIG_SCSI_U14_34F=m
+# CONFIG_SCSI_U14_34F_LINKED_COMMANDS is not set
+CONFIG_SCSI_U14_34F_MAX_TAGS=8
+CONFIG_SCSI_ULTRASTOR=m
+CONFIG_SCSI_NSP32=m
+CONFIG_SCSI_DEBUG=m
+
+#
+# Fusion MPT device support
+#
+# CONFIG_FUSION is not set
+# CONFIG_FUSION_BOOT is not set
+# CONFIG_FUSION_ISENSE is not set
+# CONFIG_FUSION_CTL is not set
+# CONFIG_FUSION_LAN is not set
+
+#
+# IEEE 1394 (FireWire) support (EXPERIMENTAL)
+#
+# CONFIG_IEEE1394 is not set
+
+#
+# I2O device support
+#
+# CONFIG_I2O is not set
+# CONFIG_I2O_PCI is not set
+# CONFIG_I2O_BLOCK is not set
+# CONFIG_I2O_LAN is not set
+# CONFIG_I2O_SCSI is not set
+# CONFIG_I2O_PROC is not set
+
+#
+# Network device support
+#
+CONFIG_NETDEVICES=y
+
+#
+# ARCnet devices
+#
+CONFIG_ARCNET=m
+CONFIG_ARCNET_1201=m
+CONFIG_ARCNET_1051=m
+CONFIG_ARCNET_RAW=m
+CONFIG_ARCNET_COM90xx=m
+CONFIG_ARCNET_COM90xxIO=m
+CONFIG_ARCNET_RIM_I=m
+CONFIG_ARCNET_COM20020=m
+# CONFIG_ARCNET_COM20020_ISA is not set
+CONFIG_ARCNET_COM20020_PCI=m
+# CONFIG_DUMMY is not set
+# CONFIG_BONDING is not set
+# CONFIG_EQUALIZER is not set
+# CONFIG_TUN is not set
+# CONFIG_ETHERTAP is not set
+
+#
+# Ethernet (10 or 100Mbit)
+#
+CONFIG_NET_ETHERNET=y
+# CONFIG_SUNLANCE is not set
+# CONFIG_HAPPYMEAL is not set
+# CONFIG_SUNBMAC is not set
+# CONFIG_SUNQE is not set
+# CONFIG_SUNGEM is not set
+# CONFIG_NET_VENDOR_3COM is not set
+# CONFIG_LANCE is not set
+# CONFIG_NET_VENDOR_SMC is not set
+# CONFIG_NET_VENDOR_RACAL is not set
+# CONFIG_HP100 is not set
+# CONFIG_NET_ISA is not set
+CONFIG_NET_PCI=y
+# CONFIG_PCNET32 is not set
+# CONFIG_AMD8111_ETH is not set
+# CONFIG_ADAPTEC_STARFIRE is not set
+# CONFIG_APRICOT is not set
+# CONFIG_B44 is not set
+# CONFIG_CS89x0 is not set
+# CONFIG_TULIP is not set
+# CONFIG_DE4X5 is not set
+# CONFIG_DGRS is not set
+# CONFIG_DM9102 is not set
+# CONFIG_EEPRO100 is not set
+# CONFIG_EEPRO100_PIO is not set
+# CONFIG_E100 is not set
+# CONFIG_LNE390 is not set
+# CONFIG_FEALNX is not set
+# CONFIG_NATSEMI is not set
+# CONFIG_NE2K_PCI is not set
+# CONFIG_FORCEDETH is not set
+# CONFIG_NE3210 is not set
+# CONFIG_ES3210 is not set
+CONFIG_8139CP=m
+CONFIG_8139TOO=m
+# CONFIG_8139TOO_PIO is not set
+CONFIG_8139TOO_TUNE_TWISTER=y
+# CONFIG_8139TOO_8129 is not set
+# CONFIG_8139_OLD_RX_RESET is not set
+# CONFIG_SIS900 is not set
+# CONFIG_EPIC100 is not set
+# CONFIG_SUNDANCE is not set
+# CONFIG_SUNDANCE_MMIO is not set
+# CONFIG_TLAN is not set
+# CONFIG_VIA_RHINE is not set
+# CONFIG_VIA_RHINE_MMIO is not set
+# CONFIG_WINBOND_840 is not set
+# CONFIG_NET_POCKET is not set
+
+#
+# Ethernet (1000 Mbit)
+#
+# CONFIG_ACENIC is not set
+# CONFIG_DL2K is not set
+# CONFIG_E1000 is not set
+# CONFIG_MYRI_SBUS is not set
+# CONFIG_NS83820 is not set
+# CONFIG_HAMACHI is not set
+# CONFIG_YELLOWFIN is not set
+# CONFIG_R8169 is not set
+# CONFIG_SK98LIN is not set
+# CONFIG_TIGON3 is not set
+# CONFIG_FDDI is not set
+# CONFIG_HIPPI is not set
+# CONFIG_PLIP is not set
+# CONFIG_PPP is not set
+# CONFIG_SLIP is not set
+
+#
+# Wireless LAN (non-hamradio)
+#
+# CONFIG_NET_RADIO is not set
+
+#
+# Token Ring devices
+#
+# CONFIG_TR is not set
+# CONFIG_NET_FC is not set
+# CONFIG_RCPCI is not set
+# CONFIG_SHAPER is not set
+
+#
+# Wan interfaces
+#
+# CONFIG_WAN is not set
+
+#
+# Amateur Radio support
+#
+# CONFIG_HAMRADIO is not set
+
+#
+# IrDA (infrared) support
+#
+# CONFIG_IRDA is not set
+
+#
+# ISDN subsystem
+#
+# CONFIG_ISDN is not set
+
+#
+# Input core support
+#
+# CONFIG_INPUT is not set
+# CONFIG_INPUT_KEYBDEV is not set
+# CONFIG_INPUT_MOUSEDEV is not set
+# CONFIG_INPUT_JOYDEV is not set
+# CONFIG_INPUT_EVDEV is not set
+# CONFIG_INPUT_UINPUT is not set
+
+#
+# Character devices
+#
+CONFIG_VT=y
+CONFIG_VT_CONSOLE=y
+CONFIG_SERIAL=y
+CONFIG_SERIAL_CONSOLE=y
+# CONFIG_SERIAL_EXTENDED is not set
+# CONFIG_SERIAL_NONSTANDARD is not set
+CONFIG_UNIX98_PTYS=y
+CONFIG_UNIX98_PTY_COUNT=512
+
+#
+# I2C support
+#
+# CONFIG_I2C is not set
+
+#
+# Mice
+#
+# CONFIG_BUSMOUSE is not set
+CONFIG_MOUSE=y
+CONFIG_PSMOUSE=y
+# CONFIG_82C710_MOUSE is not set
+# CONFIG_PC110_PAD is not set
+# CONFIG_MK712_MOUSE is not set
+
+#
+# Joysticks
+#
+# CONFIG_INPUT_GAMEPORT is not set
+
+#
+# Input core support is needed for gameports
+#
+
+#
+# Input core support is needed for joysticks
+#
+# CONFIG_QIC02_TAPE is not set
+# CONFIG_IPMI_HANDLER is not set
+# CONFIG_IPMI_PANIC_EVENT is not set
+# CONFIG_IPMI_DEVICE_INTERFACE is not set
+# CONFIG_IPMI_KCS is not set
+# CONFIG_IPMI_WATCHDOG is not set
+
+#
+# Watchdog Cards
+#
+# CONFIG_WATCHDOG is not set
+# CONFIG_SCx200 is not set
+# CONFIG_SCx200_GPIO is not set
+# CONFIG_AMD_RNG is not set
+# CONFIG_INTEL_RNG is not set
+# CONFIG_HW_RANDOM is not set
+# CONFIG_AMD_PM768 is not set
+# CONFIG_NVRAM is not set
+CONFIG_RTC=y
+# CONFIG_DTLK is not set
+# CONFIG_R3964 is not set
+# CONFIG_APPLICOM is not set
+# CONFIG_SONYPI is not set
+
+#
+# Ftape, the floppy tape device driver
+#
+# CONFIG_FTAPE is not set
+# CONFIG_AGP is not set
+
+#
+# Direct Rendering Manager (XFree86 DRI support)
+#
+# CONFIG_DRM is not set
+# CONFIG_MWAVE is not set
+# CONFIG_OBMOUSE is not set
+
+#
+# Multimedia devices
+#
+# CONFIG_VIDEO_DEV is not set
+
+#
+# File systems
+#
+CONFIG_QUOTA=y
+# CONFIG_QFMT_V2 is not set
+# CONFIG_AUTOFS_FS is not set
+# CONFIG_AUTOFS4_FS is not set
+# CONFIG_REISERFS_FS is not set
+# CONFIG_REISERFS_CHECK is not set
+# CONFIG_REISERFS_PROC_INFO is not set
+# CONFIG_ADFS_FS is not set
+# CONFIG_ADFS_FS_RW is not set
+# CONFIG_AFFS_FS is not set
+# CONFIG_HFS_FS is not set
+# CONFIG_HFSPLUS_FS is not set
+# CONFIG_BEFS_FS is not set
+# CONFIG_BEFS_DEBUG is not set
+# CONFIG_BFS_FS is not set
+CONFIG_EXT3_FS=y
+CONFIG_JBD=y
+# CONFIG_JBD_DEBUG is not set
+# CONFIG_FAT_FS is not set
+# CONFIG_MSDOS_FS is not set
+# CONFIG_UMSDOS_FS is not set
+# CONFIG_VFAT_FS is not set
+# CONFIG_EFS_FS is not set
+# CONFIG_JFFS_FS is not set
+# CONFIG_JFFS2_FS is not set
+# CONFIG_CRAMFS is not set
+CONFIG_TMPFS=y
+CONFIG_RAMFS=y
+CONFIG_ISO9660_FS=y
+# CONFIG_JOLIET is not set
+CONFIG_ZISOFS=y
+# CONFIG_JFS_FS is not set
+# CONFIG_JFS_DEBUG is not set
+# CONFIG_JFS_STATISTICS is not set
+# CONFIG_MINIX_FS is not set
+# CONFIG_VXFS_FS is not set
+# CONFIG_NTFS_FS is not set
+# CONFIG_NTFS_RW is not set
+# CONFIG_HPFS_FS is not set
+CONFIG_PROC_FS=y
+# CONFIG_DEVFS_FS is not set
+# CONFIG_DEVFS_MOUNT is not set
+# CONFIG_DEVFS_DEBUG is not set
+CONFIG_DEVPTS_FS=y
+# CONFIG_QNX4FS_FS is not set
+# CONFIG_QNX4FS_RW is not set
+# CONFIG_ROMFS_FS is not set
+CONFIG_EXT2_FS=y
+# CONFIG_SYSV_FS is not set
+# CONFIG_UDF_FS is not set
+# CONFIG_UDF_RW is not set
+# CONFIG_UFS_FS is not set
+# CONFIG_UFS_FS_WRITE is not set
+# CONFIG_XFS_FS is not set
+# CONFIG_XFS_QUOTA is not set
+# CONFIG_XFS_RT is not set
+# CONFIG_XFS_TRACE is not set
+# CONFIG_XFS_DEBUG is not set
+
+#
+# Network File Systems
+#
+# CONFIG_CODA_FS is not set
+# CONFIG_INTERMEZZO_FS is not set
+CONFIG_NFS_FS=y
+CONFIG_NFS_V3=y
+# CONFIG_NFS_DIRECTIO is not set
+# CONFIG_ROOT_NFS is not set
+# CONFIG_NFSD is not set
+# CONFIG_NFSD_V3 is not set
+# CONFIG_NFSD_TCP is not set
+CONFIG_SUNRPC=y
+CONFIG_LOCKD=y
+CONFIG_LOCKD_V4=y
+# CONFIG_SMB_FS is not set
+# CONFIG_NCP_FS is not set
+# CONFIG_NCPFS_PACKET_SIGNING is not set
+# CONFIG_NCPFS_IOCTL_LOCKING is not set
+# CONFIG_NCPFS_STRONG is not set
+# CONFIG_NCPFS_NFS_NS is not set
+# CONFIG_NCPFS_OS2_NS is not set
+# CONFIG_NCPFS_SMALLDOS is not set
+# CONFIG_NCPFS_NLS is not set
+# CONFIG_NCPFS_EXTRAS is not set
+CONFIG_ZISOFS_FS=y
+
+#
+# Partition Types
+#
+# CONFIG_PARTITION_ADVANCED is not set
+CONFIG_MSDOS_PARTITION=y
+# CONFIG_SMB_NLS is not set
+# CONFIG_NLS is not set
+
+#
+# Console drivers
+#
+CONFIG_VGA_CONSOLE=y
+CONFIG_VIDEO_SELECT=y
+# CONFIG_MDA_CONSOLE is not set
+
+#
+# Frame-buffer support
+#
+# CONFIG_FB is not set
+
+#
+# Sound
+#
+# CONFIG_SOUND is not set
+
+#
+# USB support
+#
+# CONFIG_USB is not set
+
+#
+# Support for USB gadgets
+#
+# CONFIG_USB_GADGET is not set
+
+#
+# Bluetooth support
+#
+# CONFIG_BLUEZ is not set
+
+#
+# Kernel hacking
+#
+CONFIG_DEBUG_KERNEL=y
+# CONFIG_DEBUG_STACKOVERFLOW is not set
+# CONFIG_DEBUG_HIGHMEM is not set
+# CONFIG_DEBUG_SLAB is not set
+# CONFIG_DEBUG_IOVIRT is not set
+CONFIG_MAGIC_SYSRQ=y
+# CONFIG_DEBUG_SPINLOCK is not set
+# CONFIG_FRAME_POINTER is not set
+CONFIG_LOG_BUF_SHIFT=0
+
+#
+# Cryptographic options
+#
+CONFIG_CRYPTO=y
+CONFIG_CRYPTO_HMAC=y
+CONFIG_CRYPTO_NULL=m
+CONFIG_CRYPTO_MD4=m
+CONFIG_CRYPTO_MD5=m
+CONFIG_CRYPTO_SHA1=m
+CONFIG_CRYPTO_SHA256=m
+CONFIG_CRYPTO_SHA512=m
+# CONFIG_CRYPTO_WP512 is not set
+CONFIG_CRYPTO_DES=m
+CONFIG_CRYPTO_BLOWFISH=m
+CONFIG_CRYPTO_TWOFISH=m
+CONFIG_CRYPTO_SERPENT=m
+CONFIG_CRYPTO_AES=m
+CONFIG_CRYPTO_CAST5=m
+CONFIG_CRYPTO_CAST6=m
+# CONFIG_CRYPTO_TEA is not set
+# CONFIG_CRYPTO_KHAZAD is not set
+# CONFIG_CRYPTO_ANUBIS is not set
+CONFIG_CRYPTO_ARC4=m
+CONFIG_CRYPTO_DEFLATE=m
+# CONFIG_CRYPTO_MICHAEL_MIC is not set
+CONFIG_CRYPTO_TEST=m
+
+#
+# Library routines
+#
+CONFIG_CRC32=m
+CONFIG_ZLIB_INFLATE=y
+CONFIG_ZLIB_DEFLATE=m
diff -rbNu linux-2.4.31/drivers/scsi/53c8xx_d.h linux-2.4.31-rtl3.1/drivers/scsi/53c8xx_d.h
--- linux-2.4.31/drivers/scsi/53c8xx_d.h	1999-12-09 00:17:55.000000000 +0100
+++ linux-2.4.31-rtl3.1/drivers/scsi/53c8xx_d.h	2008-01-17 09:18:45.132542500 +0100
@@ -447,10 +447,8 @@
 ABSOLUTE int_norm_aborted =0x02040000		; Aborted *dsa
 ABSOLUTE int_norm_reset = 0x02050000		; Generated BUS reset.
 ABSOLUTE int_debug_break = 0x03000000		; Break point
-
 ABSOLUTE int_debug_panic = 0x030b0000		; Panic driver
 
-
 ABSOLUTE int_test_1 = 0x04000000		; Test 1 complete
 ABSOLUTE int_test_2 = 0x04010000		; Test 2 complete
 ABSOLUTE int_test_3 = 0x04020000		; Test 3 complete
@@ -460,7 +458,6 @@
 ; each one.
 
 
-						
 ABSOLUTE NCR53c7xx_msg_abort = 0	; Pointer to abort message
 ABSOLUTE NCR53c7xx_msg_reject = 0       ; Pointer to reject message
 ABSOLUTE NCR53c7xx_zero	= 0		; long with zero in it, use for source
diff -rbNu linux-2.4.31/include/asm-i386/apic.h linux-2.4.31-rtl3.1/include/asm-i386/apic.h
--- linux-2.4.31/include/asm-i386/apic.h	2004-11-17 12:54:22.000000000 +0100
+++ linux-2.4.31-rtl3.1/include/asm-i386/apic.h	2008-01-17 09:18:45.132542500 +0100
@@ -52,6 +52,9 @@
 
 static inline void ack_APIC_irq(void)
 {
+#ifdef CONFIG_RTLINUX
+ 	label_ack_APIC(); /* used to generate RTLinux patch point */
+#endif
 	/*
 	 * ack_APIC_irq() actually gets compiled as a single instruction:
 	 * - a single rmw on Pentium/82489DX
diff -rbNu linux-2.4.31/include/asm-i386/floppy.h linux-2.4.31-rtl3.1/include/asm-i386/floppy.h
--- linux-2.4.31/include/asm-i386/floppy.h	2003-06-13 16:51:38.000000000 +0200
+++ linux-2.4.31-rtl3.1/include/asm-i386/floppy.h	2008-01-17 09:18:45.148128500 +0100
@@ -30,9 +30,11 @@
 #define SW fd_routine[use_virtual_dma&1]
 #define CSW fd_routine[can_use_virtual_dma & 1]
 
-
-#define fd_inb(port)			inb_p(port)
-#define fd_outb(port,value)		outb_p(port,value)
+/* lets see if floppies can live with outb/inb....hofrat */
+//#define fd_inb(port)			inb_p(port)
+//#define fd_outb(port,value)		outb_p(port,value)
+#define fd_inb(port)			inb(port)
+#define fd_outb(port,value)		outb(port,value)
 
 #define fd_request_dma()        CSW._request_dma(FLOPPY_DMA,"floppy")
 #define fd_free_dma()           CSW._free_dma(FLOPPY_DMA)
diff -rbNu linux-2.4.31/include/asm-i386/hw_irq.h linux-2.4.31-rtl3.1/include/asm-i386/hw_irq.h
--- linux-2.4.31/include/asm-i386/hw_irq.h	2003-08-25 13:44:43.000000000 +0200
+++ linux-2.4.31-rtl3.1/include/asm-i386/hw_irq.h	2008-01-17 09:18:45.163714500 +0100
@@ -15,6 +15,7 @@
 #include <linux/config.h>
 #include <asm/atomic.h>
 #include <asm/irq.h>
+#include <asm/rtlinux.h>
 
 /*
  * IDT vectors usable for external interrupt sources start
@@ -126,41 +127,39 @@
 #define BUILD_SMP_INTERRUPT(x,v) XBUILD_SMP_INTERRUPT(x,v)
 #define XBUILD_SMP_INTERRUPT(x,v)\
 asmlinkage void x(void); \
-asmlinkage void call_##x(void); \
 __asm__( \
 "\n"__ALIGN_STR"\n" \
 SYMBOL_NAME_STR(x) ":\n\t" \
-	"pushl $"#v"-256\n\t" \
+	"pushl $"#v"\n\t" \
 	SAVE_ALL \
-	SYMBOL_NAME_STR(call_##x)":\n\t" \
-	"call "SYMBOL_NAME_STR(smp_##x)"\n\t" \
+	"pushl $ret_from_intr\n\t" \
+        RTLINUX_LABEL(smp_##x)\
+        "jmp "SYMBOL_NAME_STR(smp_##x)"\n\t" \
 	"jmp ret_from_intr\n");
 
 #define BUILD_SMP_TIMER_INTERRUPT(x,v) XBUILD_SMP_TIMER_INTERRUPT(x,v)
 #define XBUILD_SMP_TIMER_INTERRUPT(x,v) \
 asmlinkage void x(struct pt_regs * regs); \
-asmlinkage void call_##x(void); \
 __asm__( \
 "\n"__ALIGN_STR"\n" \
 SYMBOL_NAME_STR(x) ":\n\t" \
-	"pushl $"#v"-256\n\t" \
+	"pushl $"#v"\n\t" \
 	SAVE_ALL \
 	"movl %esp,%eax\n\t" \
 	"pushl %eax\n\t" \
-	SYMBOL_NAME_STR(call_##x)":\n\t" \
+        RTLINUX_LABEL(smp_##x)\
 	"call "SYMBOL_NAME_STR(smp_##x)"\n\t" \
 	"addl $4,%esp\n\t" \
 	"jmp ret_from_intr\n");
 
 #define BUILD_COMMON_IRQ() \
-asmlinkage void call_do_IRQ(void); \
 __asm__( \
 	"\n" __ALIGN_STR"\n" \
 	"common_interrupt:\n\t" \
 	SAVE_ALL \
-	SYMBOL_NAME_STR(call_do_IRQ)":\n\t" \
-	"call " SYMBOL_NAME_STR(do_IRQ) "\n\t" \
-	"jmp ret_from_intr\n");
+ 	"pushl $ret_from_intr\n\t" \
+        RTLINUX_LABEL(do_IRQ)"\n\t" \
+	"jmp "SYMBOL_NAME_STR(do_IRQ));
 
 /* 
  * subtle. orig_eax is used by the signal code to distinct between
diff -rbNu linux-2.4.31/include/asm-i386/irq.h linux-2.4.31-rtl3.1/include/asm-i386/irq.h
--- linux-2.4.31/include/asm-i386/irq.h	2002-08-03 02:39:45.000000000 +0200
+++ linux-2.4.31-rtl3.1/include/asm-i386/irq.h	2008-01-17 09:18:45.179300500 +0100
@@ -23,7 +23,7 @@
  * Since vectors 0x00-0x1f are used/reserved for the CPU,
  * the usable vector space is 0x20-0xff (224 vectors)
  */
-#ifdef CONFIG_X86_IO_APIC
+#if defined( CONFIG_X86_IO_APIC ) || defined ( CONFIG_RTLINUX )
 #define NR_IRQS 224
 #else
 #define NR_IRQS 16
@@ -37,6 +37,7 @@
 extern void disable_irq(unsigned int);
 extern void disable_irq_nosync(unsigned int);
 extern void enable_irq(unsigned int);
+struct task_struct;
 extern void release_x86_irqs(struct task_struct *);
 
 #ifdef CONFIG_X86_LOCAL_APIC
diff -rbNu linux-2.4.31/include/asm-i386/pgalloc.h linux-2.4.31-rtl3.1/include/asm-i386/pgalloc.h
--- linux-2.4.31/include/asm-i386/pgalloc.h	2003-08-25 13:44:43.000000000 +0200
+++ linux-2.4.31-rtl3.1/include/asm-i386/pgalloc.h	2008-01-17 09:18:45.179300500 +0100
@@ -158,6 +158,33 @@
 
 extern int do_check_pgt_cache(int, int);
 
+extern inline void set_pgdir(unsigned long address, pgd_t entry)
+{
+	struct task_struct * p;
+	pgd_t *pgd;
+#ifdef CONFIG_SMP
+	int i;
+#endif
+
+	read_lock(&tasklist_lock);
+	for_each_task(p) {
+		if (!p->mm)
+			continue;
+		*pgd_offset(p->mm,address) = entry;
+	}
+	read_unlock(&tasklist_lock);
+#ifndef CONFIG_SMP
+	for (pgd = (pgd_t *)pgd_quicklist; pgd; pgd = (pgd_t *)*(unsigned long *)pgd)
+		pgd[address >> PGDIR_SHIFT] = entry;
+#else
+	/* To pgd_alloc/pgd_free, one holds master kernel lock and so does our callee, so we can
+	   modify pgd caches of other CPUs as well. -jj */
+	for (i = 0; i < NR_CPUS; i++)
+		for (pgd = (pgd_t *)cpu_data[i].pgd_quick; pgd; pgd = (pgd_t *)*(unsigned long *)pgd)
+			pgd[address >> PGDIR_SHIFT] = entry;
+#endif
+}
+
 /*
  * TLB flushing:
  *
diff -rbNu linux-2.4.31/include/asm-i386/rtlinux.h linux-2.4.31-rtl3.1/include/asm-i386/rtlinux.h
--- linux-2.4.31/include/asm-i386/rtlinux.h	1970-01-01 01:00:00.000000000 +0100
+++ linux-2.4.31-rtl3.1/include/asm-i386/rtlinux.h	2008-01-17 09:18:45.194886500 +0100
@@ -0,0 +1,14 @@
+
+/* RTLinux support for patching the running kernel.
+	* (c) Victor Yodaiken 1999
+*/
+
+#define label_ack_APIC() xlabel_code(0x12344321)
+#define xlabel_code(x) asm __volatile__(".section rtlinux_patch,\"a\"\n .align 4\n"\
+                             ".long 1f,"#x"\n" ".previous\n 1:\n" );
+#define RTLINUX_LABEL(x)\
+  ".section rtlinux_patch,\"a\"\n .align 4\n .long 1f,"#x"\n .previous\n 1:\n"
+
+#define RTLINUX_EXPORT(x)\
+  asm (".section rtlinux_funcs,\"a\"\n .align 4\n"\
+                             ".long "#x"\n .previous\n" );
diff -rbNu linux-2.4.31/include/asm-i386/rtlinux_cli.h linux-2.4.31-rtl3.1/include/asm-i386/rtlinux_cli.h
--- linux-2.4.31/include/asm-i386/rtlinux_cli.h	1970-01-01 01:00:00.000000000 +0100
+++ linux-2.4.31-rtl3.1/include/asm-i386/rtlinux_cli.h	2008-01-17 09:18:45.194886500 +0100
@@ -0,0 +1,43 @@
+#ifndef RTLINUX_CLI_H
+#define RTLINUX_CLI_H
+
+/* interrupt control.. */
+#define rtl_hard_save_flags_kernel(x)		__asm__ __volatile__("pushfl ; popl %0":"=g" (x): /* no input */ :"memory")
+#define rtl_hard_restore_flags_kernel(x) 	__asm__ __volatile__("pushl %0 ; popfl": /* no output */ :"g" (x):"memory")
+#define rtl_hard_cli_kernel() 		__asm__ __volatile__("cli": : :"memory")
+#define rtl_hard_sti_kernel()			__asm__ __volatile__("sti": : :"memory")
+
+/* For spinlocks etc */
+#define rtl_hard_local_irq_save_kernel(x)	__asm__ __volatile__("pushfl ; popl %0 ; cli":"=g" (x): /* no input */ :"memory")
+#define rtl_hard_local_irq_restore_kernel(x)	__asm__ __volatile__("pushl %0 ; popfl": /* no output */ :"g" (x):"memory")
+#define rtl_hard_local_irq_disable_kernel()	__asm__ __volatile__("cli": : :"memory")
+#define rtl_hard_local_irq_enable_kernel()	__asm__ __volatile__("sti": : :"memory")
+struct irq_control_s {
+	void (*do_save_flags)(unsigned long *);
+	void (*do_restore_flags)(unsigned long);
+	void (*do_cli)(void);
+	void (*do_sti)(void);
+	void (*do_local_irq_save)(unsigned long *);
+	void (*do_local_irq_restore)(unsigned long);
+};
+extern struct irq_control_s irq_control;
+#undef __cli
+#undef __sti
+#undef __save_flags
+#undef __restore_flags
+#undef local_irq_save
+#undef local_irq_restore
+#undef local_irq_disable
+#undef local_irq_enable
+#define __save_flags(x)	irq_control.do_save_flags(&x)
+#define __restore_flags(x) irq_control.do_restore_flags(x)
+#define __cli()		irq_control.do_cli()
+#define __sti()		irq_control.do_sti()
+#define local_irq_save(x)	irq_control.do_local_irq_save((&x))
+#define local_irq_restore(x)	irq_control.do_local_irq_restore(x)
+#define local_irq_disable()	irq_control.do_cli()
+#define local_irq_enable()	irq_control.do_sti()
+
+#undef safe_halt
+#define safe_halt()	do { __asm__ __volatile__("cli; nop; sti; hlt": : :"memory"); irq_control.do_sti();  } while (0)
+#endif /* RTLINUX_CLI_H */
diff -rbNu linux-2.4.31/include/asm-i386/smp.h linux-2.4.31-rtl3.1/include/asm-i386/smp.h
--- linux-2.4.31/include/asm-i386/smp.h	2004-11-17 12:54:22.000000000 +0100
+++ linux-2.4.31-rtl3.1/include/asm-i386/smp.h	2008-01-17 09:18:45.194886500 +0100
@@ -11,6 +11,7 @@
 #endif
 
 #ifdef CONFIG_X86_LOCAL_APIC
+#include <asm/rtlinux.h>
 #ifndef __ASSEMBLY__
 #include <asm/fixmap.h>
 #include <asm/bitops.h>
@@ -68,6 +69,9 @@
 extern volatile int cpu_to_logical_apicid[NR_CPUS];
 extern volatile int logical_apicid_to_cpu[MAX_APICID];
 
+extern volatile int physical_apicid_2_cpu[];
+#define hw_smp_processor_id() (physical_apicid_2_cpu[hard_smp_processor_id()])
+
 /*
  * General functions that each host system must provide.
  */
@@ -111,5 +115,7 @@
  
 #define PROC_CHANGE_PENALTY	15		/* Schedule penalty */
 
+#else
+#define hw_smp_processor_id() (0)
 #endif
 #endif
diff -rbNu linux-2.4.31/include/asm-i386/system.h linux-2.4.31-rtl3.1/include/asm-i386/system.h
--- linux-2.4.31/include/asm-i386/system.h	2004-04-14 15:05:40.000000000 +0200
+++ linux-2.4.31-rtl3.1/include/asm-i386/system.h	2008-01-17 09:18:45.210472500 +0100
@@ -5,6 +5,7 @@
 #include <linux/kernel.h>
 #include <linux/init.h>
 #include <asm/segment.h>
+#include <asm/rtlinux.h>
 #include <linux/bitops.h> /* for LOCK_PREFIX */
 
 #ifdef __KERNEL__
@@ -336,6 +337,9 @@
 #define local_irq_disable()	__cli()
 #define local_irq_enable()	__sti()
 
+#ifdef CONFIG_RTLINUX
+#include <asm/rtlinux_cli.h>
+#endif
 #ifdef CONFIG_SMP
 
 extern void __global_cli(void);
diff -rbNu linux-2.4.31/kernel/exit.c linux-2.4.31-rtl3.1/kernel/exit.c
--- linux-2.4.31/kernel/exit.c	2002-11-29 00:53:15.000000000 +0100
+++ linux-2.4.31-rtl3.1/kernel/exit.c	2008-01-17 09:18:45.226058500 +0100
@@ -422,10 +422,14 @@
 	write_unlock_irq(&tasklist_lock);
 }
 
+void (*rtl_do_exit_handler)(long code) = 0;
 NORET_TYPE void do_exit(long code)
 {
 	struct task_struct *tsk = current;
 
+	if (rtl_do_exit_handler) {
+		rtl_do_exit_handler(code);
+	}
 	if (in_interrupt())
 		panic("Aiee, killing interrupt handler!");
 	if (!tsk->pid)
diff -rbNu linux-2.4.31/kernel/fork.c linux-2.4.31-rtl3.1/kernel/fork.c
--- linux-2.4.31/kernel/fork.c	2005-01-19 15:10:13.000000000 +0100
+++ linux-2.4.31-rtl3.1/kernel/fork.c	2008-01-17 09:18:45.241644500 +0100
@@ -43,8 +43,8 @@
 {
 	unsigned long flags;
 
+	wq_write_lock_irqsave(&q->lock, flags); //kabi
 	wait->flags &= ~WQ_FLAG_EXCLUSIVE;
-	wq_write_lock_irqsave(&q->lock, flags);
 	__add_wait_queue(q, wait);
 	wq_write_unlock_irqrestore(&q->lock, flags);
 }
@@ -53,8 +53,8 @@
 {
 	unsigned long flags;
 
+	wq_write_lock_irqsave(&q->lock, flags); //kabi
 	wait->flags |= WQ_FLAG_EXCLUSIVE;
-	wq_write_lock_irqsave(&q->lock, flags);
 	__add_wait_queue_tail(q, wait);
 	wq_write_unlock_irqrestore(&q->lock, flags);
 }
@@ -235,7 +235,9 @@
 	atomic_set(&mm->mm_count, 1);
 	init_rwsem(&mm->mmap_sem);
 	mm->page_table_lock = SPIN_LOCK_UNLOCKED;
+	lock_kernel();
 	mm->pgd = pgd_alloc(mm);
+	unlock_kernel();
 	mm->def_flags = 0;
 	if (mm->pgd)
 		return mm;
@@ -267,7 +269,9 @@
 void fastcall __mmdrop(struct mm_struct *mm)
 {
 	BUG_ON(mm == &init_mm);
+	lock_kernel();
 	pgd_free(mm->pgd);
+	unlock_kernel();
 	check_pgt_cache();
 	destroy_context(mm);
 	free_mm(mm);
diff -rbNu linux-2.4.31/mm/vmalloc.c linux-2.4.31-rtl3.1/mm/vmalloc.c
--- linux-2.4.31/mm/vmalloc.c	2004-04-14 15:05:41.000000000 +0200
+++ linux-2.4.31-rtl3.1/mm/vmalloc.c	2008-01-17 09:18:45.257230500 +0100
@@ -166,6 +166,7 @@
 	spin_lock(&init_mm.page_table_lock);
 	do {
 		pmd_t *pmd;
+		pgd_t olddir = *dir;
 		
 		pmd = pmd_alloc(&init_mm, dir, address);
 		if (!pmd)
@@ -173,6 +174,10 @@
 
 		if (alloc_area_pmd(pmd, address, end - address, gfp_mask, prot, pages))
 			goto err;	// The kernel NEVER reclaims pmds, so no need to undo pmd_alloc() here
+#if defined(__i386__)
+		if (pgd_val(olddir) != pgd_val(*dir))
+			set_pgdir(address, *dir);
+#endif /* __i386__ */
 
 		address = (address + PGDIR_SIZE) & PGDIR_MASK;
 		dir++;
diff -rbNu linux-2.4.31/net/core/scm.c linux-2.4.31-rtl3.1/net/core/scm.c
--- linux-2.4.31/net/core/scm.c	2005-01-19 15:10:13.000000000 +0100
+++ linux-2.4.31-rtl3.1/net/core/scm.c	2008-01-17 09:18:45.272816500 +0100
@@ -105,6 +105,10 @@
 			fput(fpl->fp[i]);
 		kfree(fpl);
 	}
+#ifdef CONFIG_NET_TLBFLUSH
+	__asm__ __volatile__("invlpg %0": :"m" (*(char *)&scm));
+//	__asm__ __volatile__("invlpg %0": :"m" (*(char *)&fpl));
+#endif
 }
 
 int __scm_send(struct socket *sock, struct msghdr *msg, struct scm_cookie *p)
diff -rbNu linux-2.4.31/net/socket.c linux-2.4.31-rtl3.1/net/socket.c
--- linux-2.4.31/net/socket.c	2005-01-19 15:10:14.000000000 +0100
+++ linux-2.4.31-rtl3.1/net/socket.c	2008-01-17 09:18:45.288402500 +0100
@@ -507,6 +507,10 @@
 		err = sock->ops->sendmsg(sock, msg, size, &scm);
 		scm_destroy(&scm);
 	}
+#ifdef CONFIG_NET_TLBFLUSH
+//	__asm__ __volatile__("invlpg %0": :"m" (*(char *)&(sock->ops->sendmsg)));
+	__asm__ __volatile__("invlpg %0": :"m" (*(char *)&msg));
+#endif
 	return err;
 }
 
@@ -520,6 +524,10 @@
 	if (size >= 0)
 		scm_recv(sock, msg, &scm, flags);
 
+#ifdef CONFIG_NET_TLBFLUSH
+//	__asm__ __volatile__("invlpg %0": :"m" (*(char *)&(sock->ops->recvmsg)));
+	__asm__ __volatile__("invlpg %0": :"m" (*(char *)&msg));
+#endif
 	return size;
 }
 
